# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `specfun` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

log_shifted = utils._EngineState._get_symbol('s01baf')
log_shifted.restype = utils._EngineFloat64ScalarType.c_type
log_shifted.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

exp_complex = None # Not supported: com-valued function

tan = utils._EngineState._get_symbol('s07aaf')
tan.restype = utils._EngineFloat64ScalarType.c_type
tan.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

arcsin = utils._EngineState._get_symbol('s09aaf')
arcsin.restype = utils._EngineFloat64ScalarType.c_type
arcsin.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

arccos = utils._EngineState._get_symbol('s09abf')
arccos.restype = utils._EngineFloat64ScalarType.c_type
arccos.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

tanh = utils._EngineState._get_symbol('s10aaf')
tanh.restype = utils._EngineFloat64ScalarType.c_type
tanh.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

sinh = utils._EngineState._get_symbol('s10abf')
sinh.restype = utils._EngineFloat64ScalarType.c_type
sinh.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

cosh = utils._EngineState._get_symbol('s10acf')
cosh.restype = utils._EngineFloat64ScalarType.c_type
cosh.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

arctanh = utils._EngineState._get_symbol('s11aaf')
arctanh.restype = utils._EngineFloat64ScalarType.c_type
arctanh.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

arcsinh = utils._EngineState._get_symbol('s11abf')
arcsinh.restype = utils._EngineFloat64ScalarType.c_type
arcsinh.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

arccosh = utils._EngineState._get_symbol('s11acf')
arccosh.restype = utils._EngineFloat64ScalarType.c_type
arccosh.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

integral_exp = utils._EngineState._get_symbol('s13aaf')
integral_exp.restype = utils._EngineFloat64ScalarType.c_type
integral_exp.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

integral_cos = utils._EngineState._get_symbol('s13acf')
integral_cos.restype = utils._EngineFloat64ScalarType.c_type
integral_cos.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

integral_sin = utils._EngineState._get_symbol('s13adf')
integral_sin.restype = utils._EngineFloat64ScalarType.c_type
integral_sin.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

gamma = utils._EngineState._get_symbol('s14aaf')
gamma.restype = utils._EngineFloat64ScalarType.c_type
gamma.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

gamma_log_real = utils._EngineState._get_symbol('s14abf')
gamma_log_real.restype = utils._EngineFloat64ScalarType.c_type
gamma_log_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

polygamma = utils._EngineState._get_symbol('s14acf')
polygamma.restype = utils._EngineFloat64ScalarType.c_type
polygamma.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

polygamma_deriv = utils._EngineState._get_symbol('s14adf')
polygamma_deriv.restype = None
polygamma_deriv.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Output, ans[m].
    utils._BYREF_INT, # Input/Output, ifail.
)

psi_deriv_real = utils._EngineState._get_symbol('s14aef')
psi_deriv_real.restype = utils._EngineFloat64ScalarType.c_type
psi_deriv_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input/Output, ifail.
)

psi_deriv_complex = None # Not supported: com-valued function

gamma_log_complex = None # Not supported: com-valued function

gamma_log_scaled_real = utils._EngineState._get_symbol('s14ahf')
gamma_log_scaled_real.restype = utils._EngineFloat64ScalarType.c_type
gamma_log_scaled_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

gamma_vector = utils._EngineState._get_symbol('s14anf')
gamma_vector.restype = None
gamma_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

gamma_log_real_vector = utils._EngineState._get_symbol('s14apf')
gamma_log_real_vector.restype = None
gamma_log_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

gamma_incomplete = utils._EngineState._get_symbol('s14baf')
gamma_incomplete.restype = None
gamma_incomplete.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_FLOAT64, # Output, p.
    utils._BYREF_FLOAT64, # Output, q.
    utils._BYREF_INT, # Input/Output, ifail.
)

gamma_incomplete_vector = utils._EngineState._get_symbol('s14bnf')
gamma_incomplete_vector.restype = None
gamma_incomplete_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, a[n].
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Output, p[n].
    utils._EngineFloat64ArrayType.c_type, # Output, q[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

beta_log_real = utils._EngineState._get_symbol('s14cbf')
beta_log_real.restype = utils._EngineFloat64ScalarType.c_type
beta_log_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input/Output, ifail.
)

beta_incomplete = utils._EngineState._get_symbol('s14ccf')
beta_incomplete.restype = None
beta_incomplete.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Output, w.
    utils._BYREF_FLOAT64, # Output, w1.
    utils._BYREF_INT, # Input/Output, ifail.
)

beta_log_real_vector = utils._EngineState._get_symbol('s14cpf')
beta_log_real_vector.restype = None
beta_log_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, a[n].
    utils._EngineFloat64ArrayType.c_type, # Input, b[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

beta_incomplete_vector = utils._EngineState._get_symbol('s14cqf')
beta_incomplete_vector.restype = None
beta_incomplete_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, a[n].
    utils._EngineFloat64ArrayType.c_type, # Input, b[n].
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, w[n].
    utils._EngineFloat64ArrayType.c_type, # Output, w1[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

cdf_normal = utils._EngineState._get_symbol('s15abf')
cdf_normal.restype = utils._EngineFloat64ScalarType.c_type
cdf_normal.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

compcdf_normal = utils._EngineState._get_symbol('s15acf')
compcdf_normal.restype = utils._EngineFloat64ScalarType.c_type
compcdf_normal.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

erfc_real = utils._EngineState._get_symbol('s15adf')
erfc_real.restype = utils._EngineFloat64ScalarType.c_type
erfc_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

erf_real = utils._EngineState._get_symbol('s15aef')
erf_real.restype = utils._EngineFloat64ScalarType.c_type
erf_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

dawson = utils._EngineState._get_symbol('s15aff')
dawson.restype = utils._EngineFloat64ScalarType.c_type
dawson.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

erfcx_real = utils._EngineState._get_symbol('s15agf')
erfcx_real.restype = utils._EngineFloat64ScalarType.c_type
erfcx_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

cdf_normal_vector = utils._EngineState._get_symbol('s15apf')
cdf_normal_vector.restype = None
cdf_normal_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

compcdf_normal_vector = utils._EngineState._get_symbol('s15aqf')
compcdf_normal_vector.restype = None
compcdf_normal_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

erfc_real_vector = utils._EngineState._get_symbol('s15arf')
erfc_real_vector.restype = None
erfc_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

erf_real_vector = utils._EngineState._get_symbol('s15asf')
erf_real_vector.restype = None
erf_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

dawson_vector = utils._EngineState._get_symbol('s15atf')
dawson_vector.restype = None
dawson_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

erfcx_real_vector = utils._EngineState._get_symbol('s15auf')
erfcx_real_vector.restype = None
erfcx_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

erfc_complex = None # Not supported: com-valued function

erfc_complex_vector = utils._EngineState._get_symbol('s15drf')
erfc_complex_vector.restype = None
erfc_complex_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineComplex128ArrayType.c_type, # Input, z[n].
    utils._EngineComplex128ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_y0_real = utils._EngineState._get_symbol('s17acf')
bessel_y0_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_y0_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_y1_real = utils._EngineState._get_symbol('s17adf')
bessel_y1_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_y1_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_j0_real = utils._EngineState._get_symbol('s17aef')
bessel_j0_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_j0_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_j1_real = utils._EngineState._get_symbol('s17aff')
bessel_j1_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_j1_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_ai_real = utils._EngineState._get_symbol('s17agf')
airy_ai_real.restype = utils._EngineFloat64ScalarType.c_type
airy_ai_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_bi_real = utils._EngineState._get_symbol('s17ahf')
airy_bi_real.restype = utils._EngineFloat64ScalarType.c_type
airy_bi_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_ai_deriv = utils._EngineState._get_symbol('s17ajf')
airy_ai_deriv.restype = utils._EngineFloat64ScalarType.c_type
airy_ai_deriv.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_bi_deriv = utils._EngineState._get_symbol('s17akf')
airy_bi_deriv.restype = utils._EngineFloat64ScalarType.c_type
airy_bi_deriv.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_zeros = utils._EngineState._get_symbol('s17alf')
bessel_zeros.restype = None
bessel_zeros.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, mode.
    utils._BYREF_FLOAT64, # Input, rel.
    utils._EngineFloat64ArrayType.c_type, # Output, x[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_y0_real_vector = utils._EngineState._get_symbol('s17aqf')
bessel_y0_real_vector.restype = None
bessel_y0_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_y1_real_vector = utils._EngineState._get_symbol('s17arf')
bessel_y1_real_vector.restype = None
bessel_y1_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_j0_real_vector = utils._EngineState._get_symbol('s17asf')
bessel_j0_real_vector.restype = None
bessel_j0_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_j1_real_vector = utils._EngineState._get_symbol('s17atf')
bessel_j1_real_vector.restype = None
bessel_j1_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_ai_real_vector = utils._EngineState._get_symbol('s17auf')
airy_ai_real_vector.restype = None
airy_ai_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_bi_real_vector = utils._EngineState._get_symbol('s17avf')
airy_bi_real_vector.restype = None
airy_bi_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_ai_deriv_vector = utils._EngineState._get_symbol('s17awf')
airy_ai_deriv_vector.restype = None
airy_ai_deriv_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

airy_bi_deriv_vector = utils._EngineState._get_symbol('s17axf')
airy_bi_deriv_vector.restype = None
airy_bi_deriv_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_y_complex = utils._EngineState._get_symbol('s17dcf')
bessel_y_complex.restype = None
bessel_y_complex.argtypes = (
    utils._BYREF_FLOAT64, # Input, fnu.
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_STRING, # Input, scal(len=1).
    utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
    utils._BYREF_INT, # Output, nz.
    utils._EngineComplex128ArrayType.c_type, # Workspace, cwrk[n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
)

bessel_j_complex = utils._EngineState._get_symbol('s17def')
bessel_j_complex.restype = None
bessel_j_complex.argtypes = (
    utils._BYREF_FLOAT64, # Input, fnu.
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_STRING, # Input, scal(len=1).
    utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
    utils._BYREF_INT, # Output, nz.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
)

airy_ai_complex = utils._EngineState._get_symbol('s17dgf')
airy_ai_complex.restype = None
airy_ai_complex.argtypes = (
    utils._BYREF_STRING, # Input, deriv(len=1).
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_STRING, # Input, scal(len=1).
    utils._BYREF_COMPLEX128, # Output, ai.
    utils._BYREF_INT, # Output, nz.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of deriv.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
)

airy_bi_complex = utils._EngineState._get_symbol('s17dhf')
airy_bi_complex.restype = None
airy_bi_complex.argtypes = (
    utils._BYREF_STRING, # Input, deriv(len=1).
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_STRING, # Input, scal(len=1).
    utils._BYREF_COMPLEX128, # Output, bi.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of deriv.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
)

hankel_complex = utils._EngineState._get_symbol('s17dlf')
hankel_complex.restype = None
hankel_complex.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_FLOAT64, # Input, fnu.
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_STRING, # Input, scal(len=1).
    utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
    utils._BYREF_INT, # Output, nz.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
)

struve_h0 = utils._EngineState._get_symbol('s17gaf')
struve_h0.restype = utils._EngineFloat64ScalarType.c_type
struve_h0.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

struve_h1 = utils._EngineState._get_symbol('s17gbf')
struve_h1.restype = utils._EngineFloat64ScalarType.c_type
struve_h1.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k0_real = utils._EngineState._get_symbol('s18acf')
bessel_k0_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_k0_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k1_real = utils._EngineState._get_symbol('s18adf')
bessel_k1_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_k1_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i0_real = utils._EngineState._get_symbol('s18aef')
bessel_i0_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_i0_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i1_real = utils._EngineState._get_symbol('s18aff')
bessel_i1_real.restype = utils._EngineFloat64ScalarType.c_type
bessel_i1_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k0_real_vector = utils._EngineState._get_symbol('s18aqf')
bessel_k0_real_vector.restype = None
bessel_k0_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k1_real_vector = utils._EngineState._get_symbol('s18arf')
bessel_k1_real_vector.restype = None
bessel_k1_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i0_real_vector = utils._EngineState._get_symbol('s18asf')
bessel_i0_real_vector.restype = None
bessel_i0_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i1_real_vector = utils._EngineState._get_symbol('s18atf')
bessel_i1_real_vector.restype = None
bessel_i1_real_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k0_scaled = utils._EngineState._get_symbol('s18ccf')
bessel_k0_scaled.restype = utils._EngineFloat64ScalarType.c_type
bessel_k0_scaled.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k1_scaled = utils._EngineState._get_symbol('s18cdf')
bessel_k1_scaled.restype = utils._EngineFloat64ScalarType.c_type
bessel_k1_scaled.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i0_scaled = utils._EngineState._get_symbol('s18cef')
bessel_i0_scaled.restype = utils._EngineFloat64ScalarType.c_type
bessel_i0_scaled.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i1_scaled = utils._EngineState._get_symbol('s18cff')
bessel_i1_scaled.restype = utils._EngineFloat64ScalarType.c_type
bessel_i1_scaled.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k0_scaled_vector = utils._EngineState._get_symbol('s18cqf')
bessel_k0_scaled_vector.restype = None
bessel_k0_scaled_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k1_scaled_vector = utils._EngineState._get_symbol('s18crf')
bessel_k1_scaled_vector.restype = None
bessel_k1_scaled_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i0_scaled_vector = utils._EngineState._get_symbol('s18csf')
bessel_i0_scaled_vector.restype = None
bessel_i0_scaled_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_i1_scaled_vector = utils._EngineState._get_symbol('s18ctf')
bessel_i1_scaled_vector.restype = None
bessel_i1_scaled_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_k_complex = utils._EngineState._get_symbol('s18dcf')
bessel_k_complex.restype = None
bessel_k_complex.argtypes = (
    utils._BYREF_FLOAT64, # Input, fnu.
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_STRING, # Input, scal(len=1).
    utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
    utils._BYREF_INT, # Output, nz.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
)

bessel_i_complex = utils._EngineState._get_symbol('s18def')
bessel_i_complex.restype = None
bessel_i_complex.argtypes = (
    utils._BYREF_FLOAT64, # Input, fnu.
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_STRING, # Input, scal(len=1).
    utils._EngineComplex128ArrayType.c_type, # Output, cy[n].
    utils._BYREF_INT, # Output, nz.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scal.
)

struve_l0 = utils._EngineState._get_symbol('s18gaf')
struve_l0.restype = utils._EngineFloat64ScalarType.c_type
struve_l0.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

struve_l1 = utils._EngineState._get_symbol('s18gbf')
struve_l1.restype = utils._EngineFloat64ScalarType.c_type
struve_l1.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

struve_i0ml0 = utils._EngineState._get_symbol('s18gcf')
struve_i0ml0.restype = utils._EngineFloat64ScalarType.c_type
struve_i0ml0.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

struve_i1ml1 = utils._EngineState._get_symbol('s18gdf')
struve_i1ml1.restype = utils._EngineFloat64ScalarType.c_type
struve_i1ml1.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

bessel_j_seq_complex = utils._EngineState._get_symbol('s18gkf')
bessel_j_seq_complex.restype = None
bessel_j_seq_complex.argtypes = (
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_INT, # Input, nl.
    utils._EngineComplex128ArrayType.c_type, # Output, b[abs(nl)+1].
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_ber = utils._EngineState._get_symbol('s19aaf')
kelvin_ber.restype = utils._EngineFloat64ScalarType.c_type
kelvin_ber.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_bei = utils._EngineState._get_symbol('s19abf')
kelvin_bei.restype = utils._EngineFloat64ScalarType.c_type
kelvin_bei.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_ker = utils._EngineState._get_symbol('s19acf')
kelvin_ker.restype = utils._EngineFloat64ScalarType.c_type
kelvin_ker.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_kei = utils._EngineState._get_symbol('s19adf')
kelvin_kei.restype = utils._EngineFloat64ScalarType.c_type
kelvin_kei.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_ber_vector = utils._EngineState._get_symbol('s19anf')
kelvin_ber_vector.restype = None
kelvin_ber_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_bei_vector = utils._EngineState._get_symbol('s19apf')
kelvin_bei_vector.restype = None
kelvin_bei_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_ker_vector = utils._EngineState._get_symbol('s19aqf')
kelvin_ker_vector.restype = None
kelvin_ker_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

kelvin_kei_vector = utils._EngineState._get_symbol('s19arf')
kelvin_kei_vector.restype = None
kelvin_kei_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineIntArrayType.c_type, # Output, ivalid[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

fresnel_s = utils._EngineState._get_symbol('s20acf')
fresnel_s.restype = utils._EngineFloat64ScalarType.c_type
fresnel_s.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

fresnel_c = utils._EngineState._get_symbol('s20adf')
fresnel_c.restype = utils._EngineFloat64ScalarType.c_type
fresnel_c.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

fresnel_s_vector = utils._EngineState._get_symbol('s20aqf')
fresnel_s_vector.restype = None
fresnel_s_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

fresnel_c_vector = utils._EngineState._get_symbol('s20arf')
fresnel_c_vector.restype = None
fresnel_c_vector.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_symm_1_degen = utils._EngineState._get_symbol('s21baf')
ellipint_symm_1_degen.restype = utils._EngineFloat64ScalarType.c_type
ellipint_symm_1_degen.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, y.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_symm_1 = utils._EngineState._get_symbol('s21bbf')
ellipint_symm_1.restype = utils._EngineFloat64ScalarType.c_type
ellipint_symm_1.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, y.
    utils._BYREF_FLOAT64, # Input, z.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_symm_2 = utils._EngineState._get_symbol('s21bcf')
ellipint_symm_2.restype = utils._EngineFloat64ScalarType.c_type
ellipint_symm_2.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, y.
    utils._BYREF_FLOAT64, # Input, z.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_symm_3 = utils._EngineState._get_symbol('s21bdf')
ellipint_symm_3.restype = utils._EngineFloat64ScalarType.c_type
ellipint_symm_3.argtypes = (
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, y.
    utils._BYREF_FLOAT64, # Input, z.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_legendre_1 = utils._EngineState._get_symbol('s21bef')
ellipint_legendre_1.restype = utils._EngineFloat64ScalarType.c_type
ellipint_legendre_1.argtypes = (
    utils._BYREF_FLOAT64, # Input, phi.
    utils._BYREF_FLOAT64, # Input, dm.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_legendre_2 = utils._EngineState._get_symbol('s21bff')
ellipint_legendre_2.restype = utils._EngineFloat64ScalarType.c_type
ellipint_legendre_2.argtypes = (
    utils._BYREF_FLOAT64, # Input, phi.
    utils._BYREF_FLOAT64, # Input, dm.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_legendre_3 = utils._EngineState._get_symbol('s21bgf')
ellipint_legendre_3.restype = utils._EngineFloat64ScalarType.c_type
ellipint_legendre_3.argtypes = (
    utils._BYREF_FLOAT64, # Input, dn.
    utils._BYREF_FLOAT64, # Input, phi.
    utils._BYREF_FLOAT64, # Input, dm.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_complete_1 = utils._EngineState._get_symbol('s21bhf')
ellipint_complete_1.restype = utils._EngineFloat64ScalarType.c_type
ellipint_complete_1.argtypes = (
    utils._BYREF_FLOAT64, # Input, dm.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_complete_2 = utils._EngineState._get_symbol('s21bjf')
ellipint_complete_2.restype = utils._EngineFloat64ScalarType.c_type
ellipint_complete_2.argtypes = (
    utils._BYREF_FLOAT64, # Input, dm.
    utils._BYREF_INT, # Input/Output, ifail.
)

jacellip_real = utils._EngineState._get_symbol('s21caf')
jacellip_real.restype = None
jacellip_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, u.
    utils._BYREF_FLOAT64, # Input, m.
    utils._BYREF_FLOAT64, # Output, sn.
    utils._BYREF_FLOAT64, # Output, cn.
    utils._BYREF_FLOAT64, # Output, dn.
    utils._BYREF_INT, # Input/Output, ifail.
)

jacellip_complex = utils._EngineState._get_symbol('s21cbf')
jacellip_complex.restype = None
jacellip_complex.argtypes = (
    utils._BYREF_COMPLEX128, # Input, z.
    utils._BYREF_FLOAT64, # Input, ak2.
    utils._BYREF_COMPLEX128, # Output, sn.
    utils._BYREF_COMPLEX128, # Output, cn.
    utils._BYREF_COMPLEX128, # Output, dn.
    utils._BYREF_INT, # Input/Output, ifail.
)

jactheta_real = utils._EngineState._get_symbol('s21ccf')
jactheta_real.restype = utils._EngineFloat64ScalarType.c_type
jactheta_real.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, q.
    utils._BYREF_INT, # Input/Output, ifail.
)

ellipint_general_2 = None # Not supported: com-valued function

legendre_p = utils._EngineState._get_symbol('s22aaf')
legendre_p.restype = None
legendre_p.argtypes = (
    utils._BYREF_INT, # Input, mode.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, nl.
    utils._EngineFloat64ArrayType.c_type, # Output, p[nl+1].
    utils._BYREF_INT, # Input/Output, ifail.
)

hyperg_confl_real = utils._EngineState._get_symbol('s22baf')
hyperg_confl_real.restype = None
hyperg_confl_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Output, m.
    utils._BYREF_INT, # Input/Output, ifail.
)

hyperg_confl_real_scaled = utils._EngineState._get_symbol('s22bbf')
hyperg_confl_real_scaled.restype = None
hyperg_confl_real_scaled.argtypes = (
    utils._BYREF_FLOAT64, # Input, ani.
    utils._BYREF_FLOAT64, # Input, adr.
    utils._BYREF_FLOAT64, # Input, bni.
    utils._BYREF_FLOAT64, # Input, bdr.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Output, frm.
    utils._BYREF_INT, # Output, scm.
    utils._BYREF_INT, # Input/Output, ifail.
)

hyperg_gauss_real = utils._EngineState._get_symbol('s22bef')
hyperg_gauss_real.restype = utils._EngineFloat64ScalarType.c_type
hyperg_gauss_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_INT, # Input/Output, ifail.
)

hyperg_gauss_real_scaled = utils._EngineState._get_symbol('s22bff')
hyperg_gauss_real_scaled.restype = None
hyperg_gauss_real_scaled.argtypes = (
    utils._BYREF_FLOAT64, # Input, ani.
    utils._BYREF_FLOAT64, # Input, adr.
    utils._BYREF_FLOAT64, # Input, bni.
    utils._BYREF_FLOAT64, # Input, bdr.
    utils._BYREF_FLOAT64, # Input, cni.
    utils._BYREF_FLOAT64, # Input, cdr.
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Output, frf.
    utils._BYREF_INT, # Output, scf.
    utils._BYREF_INT, # Input/Output, ifail.
)

mathieu_ang_periodic_real = utils._EngineState._get_symbol('s22caf')
mathieu_ang_periodic_real.restype = None
mathieu_ang_periodic_real.argtypes = (
    utils._BYREF_INT, # Input, ordval.
    utils._BYREF_FLOAT64, # Input, q.
    utils._BYREF_INT, # Input, parity.
    utils._BYREF_INT, # Input, mode.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f[n].
    utils._EngineFloat64ArrayType.c_type, # Output, f_deriv[n].
    utils._BYREF_FLOAT64, # Output, a.
    utils._BYREF_INT, # Input/Output, ifail.
)

opt_bsm_price = utils._EngineState._get_symbol('s30aaf')
opt_bsm_price.restype = None
opt_bsm_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_bsm_greeks = utils._EngineState._get_symbol('s30abf')
opt_bsm_greeks.restype = None
opt_bsm_greeks.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_lookback_fls_price = utils._EngineState._get_symbol('s30baf')
opt_lookback_fls_price.restype = None
opt_lookback_fls_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, sm[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_lookback_fls_greeks = utils._EngineState._get_symbol('s30bbf')
opt_lookback_fls_greeks.restype = None
opt_lookback_fls_greeks.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, sm[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_binary_con_price = utils._EngineState._get_symbol('s30caf')
opt_binary_con_price.restype = None
opt_binary_con_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._BYREF_FLOAT64, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_binary_con_greeks = utils._EngineState._get_symbol('s30cbf')
opt_binary_con_greeks.restype = None
opt_binary_con_greeks.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._BYREF_FLOAT64, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_binary_aon_price = utils._EngineState._get_symbol('s30ccf')
opt_binary_aon_price.restype = None
opt_binary_aon_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_binary_aon_greeks = utils._EngineState._get_symbol('s30cdf')
opt_binary_aon_greeks.restype = None
opt_binary_aon_greeks.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_barrier_std_price = utils._EngineState._get_symbol('s30faf')
opt_barrier_std_price.restype = None
opt_barrier_std_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_STRING, # Input, type(len=2).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._BYREF_FLOAT64, # Input, h.
    utils._BYREF_FLOAT64, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of type.
)

opt_jumpdiff_merton_price = utils._EngineState._get_symbol('s30jaf')
opt_jumpdiff_merton_price.restype = None
opt_jumpdiff_merton_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, lambda.
    utils._BYREF_FLOAT64, # Input, jvol.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_jumpdiff_merton_greeks = utils._EngineState._get_symbol('s30jbf')
opt_jumpdiff_merton_greeks.restype = None
opt_jumpdiff_merton_greeks.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, lambda.
    utils._BYREF_FLOAT64, # Input, jvol.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_heston_price = utils._EngineState._get_symbol('s30naf')
opt_heston_price.restype = None
opt_heston_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigmav.
    utils._BYREF_FLOAT64, # Input, kappa.
    utils._BYREF_FLOAT64, # Input, corr.
    utils._BYREF_FLOAT64, # Input, var0.
    utils._BYREF_FLOAT64, # Input, eta.
    utils._BYREF_FLOAT64, # Input, grisk.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_heston_greeks = utils._EngineState._get_symbol('s30nbf')
opt_heston_greeks.restype = None
opt_heston_greeks.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigmav.
    utils._BYREF_FLOAT64, # Input, kappa.
    utils._BYREF_FLOAT64, # Input, corr.
    utils._BYREF_FLOAT64, # Input, var0.
    utils._BYREF_FLOAT64, # Input, eta.
    utils._BYREF_FLOAT64, # Input, grisk.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_heston_term = utils._EngineState._get_symbol('s30ncf')
opt_heston_term.restype = None
opt_heston_term.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, numts.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, fwd.
    utils._BYREF_FLOAT64, # Input, disc.
    utils._EngineFloat64ArrayType.c_type, # Input, ts[numts].
    utils._BYREF_FLOAT64, # Input, t.
    utils._EngineFloat64ArrayType.c_type, # Input, alpha[numts].
    utils._EngineFloat64ArrayType.c_type, # Input, lambda[numts].
    utils._EngineFloat64ArrayType.c_type, # Input, corr[numts].
    utils._EngineFloat64ArrayType.c_type, # Input, sigmat[numts].
    utils._BYREF_FLOAT64, # Input, var0.
    utils._EngineFloat64ArrayType.c_type, # Output, p[m].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_amer_bs_price = utils._EngineState._get_symbol('s30qcf')
opt_amer_bs_price.restype = None
opt_amer_bs_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, q.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_asian_geom_price = utils._EngineState._get_symbol('s30saf')
opt_asian_geom_price.restype = None
opt_asian_geom_price.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, b.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)

opt_asian_geom_greeks = utils._EngineState._get_symbol('s30sbf')
opt_asian_geom_greeks.restype = None
opt_asian_geom_greeks.argtypes = (
    utils._BYREF_STRING, # Input, calput(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, b.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, n].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, delta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, gamma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vega[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, rho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, crho[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vanna[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, charm[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, speed[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, colour[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, zomma[ldp, n].
    utils._EngineFloat64ArrayType.c_type, # Output, vomma[ldp, n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of calput.
)
