# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `tsa` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

uni_diff = utils._EngineState._get_symbol('g13aaf')
uni_diff.restype = None
uni_diff.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
    utils._BYREF_INT, # Input, nx.
    utils._BYREF_INT, # Input, nd.
    utils._BYREF_INT, # Input, nds.
    utils._BYREF_INT, # Input, ns.
    utils._EngineFloat64ArrayType.c_type, # Output, xd[nx].
    utils._BYREF_INT, # Output, nxd.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_autocorr = utils._EngineState._get_symbol('g13abf')
uni_autocorr.restype = None
uni_autocorr.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
    utils._BYREF_INT, # Input, nx.
    utils._BYREF_INT, # Input, nk.
    utils._BYREF_FLOAT64, # Output, xm.
    utils._BYREF_FLOAT64, # Output, xv.
    utils._EngineFloat64ArrayType.c_type, # Output, r[nk].
    utils._BYREF_FLOAT64, # Output, stat.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_autocorr_part = utils._EngineState._get_symbol('g13acf')
uni_autocorr_part.restype = None
uni_autocorr_part.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, r[nk].
    utils._BYREF_INT, # Input, nk.
    utils._BYREF_INT, # Input, nl.
    utils._EngineFloat64ArrayType.c_type, # Output, p[nl].
    utils._EngineFloat64ArrayType.c_type, # Output, v[nl].
    utils._EngineFloat64ArrayType.c_type, # Output, ar[nl].
    utils._BYREF_INT, # Output, nvl.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arima_prelim = utils._EngineState._get_symbol('g13adf')
uni_arima_prelim.restype = None
uni_arima_prelim.argtypes = (
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input, r[nk].
    utils._BYREF_INT, # Input, nk.
    utils._BYREF_FLOAT64, # Input, xv.
    utils._BYREF_INT, # Input, npar.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wa[nwa].
    utils._BYREF_INT, # Input, nwa.
    utils._EngineFloat64ArrayType.c_type, # Output, par[npar].
    utils._BYREF_FLOAT64, # Output, rv.
    utils._EngineIntArrayType.c_type, # Output, isf[4].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arima_estim_piv = utils._EngineState._sys_functype(
    None,
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_INT, # Input, kfc.
    utils._EngineIntArrayType.c_type, # Input, icount[6].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, g[igh].
    utils._EngineFloat64ArrayType.c_type, # Input, h[ldh, igh].
    utils._BYREF_INT, # Input, ldh.
    utils._BYREF_INT, # Input, igh.
    utils._BYREF_INT, # Input, itc.
    utils._EngineFloat64ArrayType.c_type, # Input, zsp[4].
)
uni_arima_estim = utils._EngineState._get_symbol('g13aef')
uni_arima_estim.restype = None
uni_arima_estim.argtypes = (
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input/Output, c.
    utils._BYREF_INT, # Input, kfc.
    utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
    utils._BYREF_INT, # Input, nx.
    utils._EngineIntArrayType.c_type, # Output, icount[6].
    utils._EngineFloat64ArrayType.c_type, # Output, ex[iex].
    utils._EngineFloat64ArrayType.c_type, # Output, exr[iex].
    utils._EngineFloat64ArrayType.c_type, # Output, al[iex].
    utils._BYREF_INT, # Input, iex.
    utils._BYREF_FLOAT64, # Output, s.
    utils._EngineFloat64ArrayType.c_type, # Output, g[igh].
    utils._BYREF_INT, # Input, igh.
    utils._EngineFloat64ArrayType.c_type, # Output, sd[igh].
    utils._EngineFloat64ArrayType.c_type, # Output, h[ldh, igh].
    utils._BYREF_INT, # Input, ldh.
    utils._EngineFloat64ArrayType.c_type, # Output, st[ist].
    utils._BYREF_INT, # Input, ist.
    utils._BYREF_INT, # Output, nst.
    uni_arima_estim_piv,
    utils._BYREF_INT, # Input, kpiv.
    utils._BYREF_INT, # Input, nit.
    utils._BYREF_INT, # Output, itc.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, zsp[4].
    utils._BYREF_INT, # Input, kzsp.
    utils._EngineIntArrayType.c_type, # Output, isf[4].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._EngineFloat64ArrayType.c_type, # Workspace, hc[ldh, igh].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arima_estim_easy = utils._EngineState._get_symbol('g13aff')
uni_arima_estim_easy.restype = None
uni_arima_estim_easy.argtypes = (
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input/Output, c.
    utils._BYREF_INT, # Input, kfc.
    utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
    utils._BYREF_INT, # Input, nx.
    utils._BYREF_FLOAT64, # Output, s.
    utils._BYREF_INT, # Output, ndf.
    utils._EngineFloat64ArrayType.c_type, # Output, sd[nppc].
    utils._BYREF_INT, # Input, nppc.
    utils._EngineFloat64ArrayType.c_type, # Output, cm[ldcm, nppc].
    utils._BYREF_INT, # Input, ldcm.
    utils._EngineFloat64ArrayType.c_type, # Output, st[nx].
    utils._BYREF_INT, # Output, nst.
    utils._BYREF_INT, # Input, kpiv.
    utils._BYREF_INT, # Input, nit.
    utils._BYREF_INT, # Output, itc.
    utils._EngineIntArrayType.c_type, # Output, isf[4].
    utils._EngineFloat64ArrayType.c_type, # Output, res[ires].
    utils._BYREF_INT, # Input, ires.
    utils._BYREF_INT, # Output, nres.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arima_estim_sample_piv = utils._EngineState._get_symbol('g13afz')
uni_arima_estim_sample_piv.restype = None
uni_arima_estim_sample_piv.argtypes = (
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_INT, # Input, kfc.
    utils._EngineIntArrayType.c_type, # Input, icount[6].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Input, g[igh].
    utils._EngineFloat64ArrayType.c_type, # Input, h[ldh, igh].
    utils._BYREF_INT, # Input, ldh.
    utils._BYREF_INT, # Input, igh.
    utils._BYREF_INT, # Input, itc.
    utils._EngineFloat64ArrayType.c_type, # Input, zsp[4].
)

uni_arima_update = utils._EngineState._get_symbol('g13agf')
uni_arima_update.restype = None
uni_arima_update.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, st[nst].
    utils._BYREF_INT, # Input, nst.
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input, c.
    utils._EngineFloat64ArrayType.c_type, # Input, anx[nuv].
    utils._BYREF_INT, # Input, nuv.
    utils._EngineFloat64ArrayType.c_type, # Output, anexr[nuv].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wa[nwa].
    utils._BYREF_INT, # Input, nwa.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arima_forecast_state = utils._EngineState._get_symbol('g13ahf')
uni_arima_forecast_state.restype = None
uni_arima_forecast_state.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, st[nst].
    utils._BYREF_INT, # Input, nst.
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_FLOAT64, # Input, rms.
    utils._BYREF_INT, # Input, nfv.
    utils._EngineFloat64ArrayType.c_type, # Output, fva[nfv].
    utils._EngineFloat64ArrayType.c_type, # Output, fsd[nfv].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wa[nwa].
    utils._BYREF_INT, # Input, nwa.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arima_forcecast = utils._EngineState._get_symbol('g13ajf')
uni_arima_forcecast.restype = None
uni_arima_forcecast.argtypes = (
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_INT, # Input, kfc.
    utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
    utils._BYREF_INT, # Input, nx.
    utils._BYREF_FLOAT64, # Output, rms.
    utils._EngineFloat64ArrayType.c_type, # Output, st[ist].
    utils._BYREF_INT, # Input, ist.
    utils._BYREF_INT, # Output, nst.
    utils._BYREF_INT, # Input, nfv.
    utils._EngineFloat64ArrayType.c_type, # Output, fva[ifv].
    utils._EngineFloat64ArrayType.c_type, # Output, fsd[ifv].
    utils._BYREF_INT, # Input, ifv.
    utils._EngineIntArrayType.c_type, # Output, isf[4].
    utils._EngineFloat64ArrayType.c_type, # Workspace, w[iw].
    utils._BYREF_INT, # Input, iw.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_smooth_exp = utils._EngineState._get_symbol('g13amf')
uni_smooth_exp.restype = None
uni_smooth_exp.argtypes = (
    utils._BYREF_INT, # Input, mode.
    utils._BYREF_INT, # Input, itype.
    utils._BYREF_INT, # Input, p.
    utils._EngineFloat64ArrayType.c_type, # Input, param[*].
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_INT, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, init[*].
    utils._BYREF_INT, # Input, nf.
    utils._EngineFloat64ArrayType.c_type, # Output, fv[nf].
    utils._EngineFloat64ArrayType.c_type, # Output, fse[nf].
    utils._EngineFloat64ArrayType.c_type, # Output, yhat[n].
    utils._EngineFloat64ArrayType.c_type, # Output, res[n].
    utils._BYREF_FLOAT64, # Output, dv.
    utils._BYREF_FLOAT64, # Output, ad.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, r[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arima_resid = utils._EngineState._get_symbol('g13asf')
uni_arima_resid.restype = None
uni_arima_resid.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, v[n].
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_INT, # Input, ishow.
    utils._EngineFloat64ArrayType.c_type, # Output, r[m].
    utils._EngineFloat64ArrayType.c_type, # Output, rcm[ldrcm, m].
    utils._BYREF_INT, # Input, ldrcm.
    utils._BYREF_FLOAT64, # Output, chi.
    utils._BYREF_INT, # Output, idf.
    utils._BYREF_FLOAT64, # Output, siglev.
    utils._EngineIntArrayType.c_type, # Workspace, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_means = utils._EngineState._get_symbol('g13auf')
uni_means.restype = None
uni_means.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, z[n].
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, ngrps.
    utils._BYREF_STRING, # Input, rs(len=1).
    utils._EngineFloat64ArrayType.c_type, # Output, y[ngrps].
    utils._EngineFloat64ArrayType.c_type, # Output, mean[ngrps].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of rs.
)

uni_dickey_fuller_unit = utils._EngineState._get_symbol('g13awf')
uni_dickey_fuller_unit.restype = utils._EngineFloat64ScalarType.c_type
uni_dickey_fuller_unit.argtypes = (
    utils._BYREF_INT, # Input, type.
    utils._BYREF_INT, # Input, p.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_filter_arima = utils._EngineState._get_symbol('g13baf')
multi_filter_arima.restype = None
multi_filter_arima.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, y[ny].
    utils._BYREF_INT, # Input, ny.
    utils._EngineIntArrayType.c_type, # Input, mr[nmr].
    utils._BYREF_INT, # Input, nmr.
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input, cy.
    utils._EngineFloat64ArrayType.c_type, # Output, wa[nwa].
    utils._BYREF_INT, # Input, nwa.
    utils._EngineFloat64ArrayType.c_type, # Output, b[nb].
    utils._BYREF_INT, # Input, nb.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_filter_transf = utils._EngineState._get_symbol('g13bbf')
multi_filter_transf.restype = None
multi_filter_transf.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, y[ny].
    utils._BYREF_INT, # Input, ny.
    utils._EngineIntArrayType.c_type, # Input, mr[nmr].
    utils._BYREF_INT, # Input, nmr.
    utils._EngineFloat64ArrayType.c_type, # Input, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._BYREF_FLOAT64, # Input, cy.
    utils._EngineFloat64ArrayType.c_type, # Output, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._EngineFloat64ArrayType.c_type, # Output, b[nb].
    utils._BYREF_INT, # Input, nb.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_xcorr = utils._EngineState._get_symbol('g13bcf')
multi_xcorr.restype = None
multi_xcorr.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, x[nxy].
    utils._EngineFloat64ArrayType.c_type, # Input, y[nxy].
    utils._BYREF_INT, # Input, nxy.
    utils._BYREF_INT, # Input, nl.
    utils._BYREF_FLOAT64, # Output, s.
    utils._BYREF_FLOAT64, # Output, r0.
    utils._EngineFloat64ArrayType.c_type, # Output, r[nl].
    utils._BYREF_FLOAT64, # Output, stat.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_transf_prelim = utils._EngineState._get_symbol('g13bdf')
multi_transf_prelim.restype = None
multi_transf_prelim.argtypes = (
    utils._BYREF_FLOAT64, # Input, r0.
    utils._EngineFloat64ArrayType.c_type, # Input, r[nl].
    utils._BYREF_INT, # Input, nl.
    utils._EngineIntArrayType.c_type, # Input, nna[3].
    utils._BYREF_FLOAT64, # Input, s.
    utils._BYREF_INT, # Input, nwds.
    utils._EngineFloat64ArrayType.c_type, # Output, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._EngineFloat64ArrayType.c_type, # Output, wds[nwds].
    utils._EngineIntArrayType.c_type, # Output, isf[2].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_inputmod_estim = utils._EngineState._get_symbol('g13bef')
multi_inputmod_estim.restype = None
multi_inputmod_estim.argtypes = (
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._BYREF_INT, # Input, nser.
    utils._EngineIntArrayType.c_type, # Input, mt[4, nser].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, para[npara].
    utils._BYREF_INT, # Input, npara.
    utils._BYREF_INT, # Input, kfc.
    utils._BYREF_INT, # Input, nxxy.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xxy[ldxxy, nser].
    utils._BYREF_INT, # Input, ldxxy.
    utils._BYREF_INT, # Input, kef.
    utils._BYREF_INT, # Input, nit.
    utils._BYREF_INT, # Input, kzsp.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, zsp[4].
    utils._BYREF_INT, # Output, itc.
    utils._EngineFloat64ArrayType.c_type, # Output, sd[npara].
    utils._EngineFloat64ArrayType.c_type, # Output, cm[ldcm, npara].
    utils._BYREF_INT, # Input, ldcm.
    utils._BYREF_FLOAT64, # Output, s.
    utils._BYREF_FLOAT64, # Output, d.
    utils._BYREF_INT, # Output, ndf.
    utils._BYREF_INT, # Input, kzef.
    utils._EngineFloat64ArrayType.c_type, # Output, res[nxxy].
    utils._EngineFloat64ArrayType.c_type, # Output, sttf[isttf].
    utils._BYREF_INT, # Input, isttf.
    utils._BYREF_INT, # Output, nsttf.
    utils._EngineFloat64ArrayType.c_type, # Output, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._EngineIntArrayType.c_type, # Output, mwa[imwa].
    utils._BYREF_INT, # Input, imwa.
    utils._BYREF_INT, # Input, kpriv.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_inputmod_update = utils._EngineState._get_symbol('g13bgf')
multi_inputmod_update.restype = None
multi_inputmod_update.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, sttf[nsttf].
    utils._BYREF_INT, # Input, nsttf.
    utils._EngineIntArrayType.c_type, # Input, mr[7].
    utils._BYREF_INT, # Input, nser.
    utils._EngineIntArrayType.c_type, # Input, mt[4, nser].
    utils._EngineFloat64ArrayType.c_type, # Input, para[npara].
    utils._BYREF_INT, # Input, npara.
    utils._BYREF_INT, # Input, nnv.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xxyn[ldxxyn, nser].
    utils._BYREF_INT, # Input, ldxxyn.
    utils._BYREF_INT, # Input, kzef.
    utils._EngineFloat64ArrayType.c_type, # Output, res[nnv].
    utils._EngineFloat64ArrayType.c_type, # Output, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_inputmod_forecast_state = utils._EngineState._get_symbol('g13bhf')
multi_inputmod_forecast_state.restype = None
multi_inputmod_forecast_state.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, sttf[nsttf].
    utils._BYREF_INT, # Input, nsttf.
    utils._EngineIntArrayType.c_type, # Input/Output, mr[7].
    utils._BYREF_INT, # Input, nser.
    utils._EngineIntArrayType.c_type, # Input, mt[4, nser].
    utils._EngineFloat64ArrayType.c_type, # Input, para[npara].
    utils._BYREF_INT, # Input, npara.
    utils._BYREF_INT, # Input, nfv.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xxyn[ldxxyn, nser].
    utils._BYREF_INT, # Input, ldxxyn.
    utils._EngineIntArrayType.c_type, # Input/Output, mrx[7, nser].
    utils._EngineFloat64ArrayType.c_type, # Input, parx[ldparx, nser].
    utils._BYREF_INT, # Input, ldparx.
    utils._EngineFloat64ArrayType.c_type, # Input, rmsxy[nser].
    utils._BYREF_INT, # Input, kzef.
    utils._EngineFloat64ArrayType.c_type, # Output, fva[nfv].
    utils._EngineFloat64ArrayType.c_type, # Output, fsd[nfv].
    utils._EngineFloat64ArrayType.c_type, # Output, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_inputmod_forecast = utils._EngineState._get_symbol('g13bjf')
multi_inputmod_forecast.restype = None
multi_inputmod_forecast.argtypes = (
    utils._EngineIntArrayType.c_type, # Input/Output, mr[7].
    utils._BYREF_INT, # Input, nser.
    utils._EngineIntArrayType.c_type, # Input, mt[4, nser].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, para[npara].
    utils._BYREF_INT, # Input, npara.
    utils._BYREF_INT, # Input, kfc.
    utils._BYREF_INT, # Input, nev.
    utils._BYREF_INT, # Input, nfv.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xxy[ldxxy, nser].
    utils._BYREF_INT, # Input, ldxxy.
    utils._BYREF_INT, # Input, kzef.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rmsxy[nser].
    utils._EngineIntArrayType.c_type, # Input/Output, mrx[7, nser].
    utils._EngineFloat64ArrayType.c_type, # Input, parx[ldparx, nser].
    utils._BYREF_INT, # Input, ldparx.
    utils._EngineFloat64ArrayType.c_type, # Output, fva[nfv].
    utils._EngineFloat64ArrayType.c_type, # Output, fsd[nfv].
    utils._EngineFloat64ArrayType.c_type, # Output, sttf[isttf].
    utils._BYREF_INT, # Input, isttf.
    utils._BYREF_INT, # Output, nsttf.
    utils._EngineFloat64ArrayType.c_type, # Output, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._EngineIntArrayType.c_type, # Output, mwa[imwa].
    utils._BYREF_INT, # Input, imwa.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_spectrum_lag = utils._EngineState._get_symbol('g13caf')
uni_spectrum_lag.restype = None
uni_spectrum_lag.argtypes = (
    utils._BYREF_INT, # Input, nx.
    utils._BYREF_INT, # Input, mtx.
    utils._BYREF_FLOAT64, # Input, px.
    utils._BYREF_INT, # Input, iw.
    utils._BYREF_INT, # Input, mw.
    utils._BYREF_INT, # Input, ic.
    utils._BYREF_INT, # Input, nc.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, c[nc].
    utils._BYREF_INT, # Input, kc.
    utils._BYREF_INT, # Input, l.
    utils._BYREF_INT, # Input, lg.
    utils._BYREF_INT, # Input, nxg.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xg[nxg].
    utils._BYREF_INT, # Output, ng.
    utils._EngineFloat64ArrayType.c_type, # Output, stats[4].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_spectrum_daniell = utils._EngineState._get_symbol('g13cbf')
uni_spectrum_daniell.restype = None
uni_spectrum_daniell.argtypes = (
    utils._BYREF_INT, # Input, nx.
    utils._BYREF_INT, # Input, mtx.
    utils._BYREF_FLOAT64, # Input, px.
    utils._BYREF_INT, # Input, mw.
    utils._BYREF_FLOAT64, # Input, pw.
    utils._BYREF_INT, # Input, l.
    utils._BYREF_INT, # Input, kc.
    utils._BYREF_INT, # Input, lg.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xg[kc].
    utils._BYREF_INT, # Output, ng.
    utils._EngineFloat64ArrayType.c_type, # Output, stats[4].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_spectrum_lag = utils._EngineState._get_symbol('g13ccf')
multi_spectrum_lag.restype = None
multi_spectrum_lag.argtypes = (
    utils._BYREF_INT, # Input, nxy.
    utils._BYREF_INT, # Input, mtxy.
    utils._BYREF_FLOAT64, # Input, pxy.
    utils._BYREF_INT, # Input, iw.
    utils._BYREF_INT, # Input, mw.
    utils._BYREF_INT, # Input, ish.
    utils._BYREF_INT, # Input, ic.
    utils._BYREF_INT, # Input, nc.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, cxy[nc].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, cyx[nc].
    utils._BYREF_INT, # Input, kc.
    utils._BYREF_INT, # Input, l.
    utils._BYREF_INT, # Input, nxyg.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xg[nxyg].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, yg[nxyg].
    utils._BYREF_INT, # Output, ng.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_spectrum_daniell = utils._EngineState._get_symbol('g13cdf')
multi_spectrum_daniell.restype = None
multi_spectrum_daniell.argtypes = (
    utils._BYREF_INT, # Input, nxy.
    utils._BYREF_INT, # Input, mtxy.
    utils._BYREF_FLOAT64, # Input, pxy.
    utils._BYREF_INT, # Input, mw.
    utils._BYREF_INT, # Input, ish.
    utils._BYREF_FLOAT64, # Input, pw.
    utils._BYREF_INT, # Input, l.
    utils._BYREF_INT, # Input, kc.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xg[kc].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, yg[kc].
    utils._BYREF_INT, # Output, ng.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_spectrum_bivar = utils._EngineState._get_symbol('g13cef')
multi_spectrum_bivar.restype = None
multi_spectrum_bivar.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, xg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, yg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, xyrg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, xyig[ng].
    utils._BYREF_INT, # Input, ng.
    utils._EngineFloat64ArrayType.c_type, # Input, stats[4].
    utils._EngineFloat64ArrayType.c_type, # Output, ca[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, calw[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, caup[ng].
    utils._BYREF_FLOAT64, # Output, t.
    utils._EngineFloat64ArrayType.c_type, # Output, sc[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, sclw[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, scup[ng].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_gain_bivar = utils._EngineState._get_symbol('g13cff')
multi_gain_bivar.restype = None
multi_gain_bivar.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, xg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, yg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, xyrg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, xyig[ng].
    utils._BYREF_INT, # Input, ng.
    utils._EngineFloat64ArrayType.c_type, # Input, stats[4].
    utils._EngineFloat64ArrayType.c_type, # Output, gn[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, gnlw[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, gnup[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, ph[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, phlw[ng].
    utils._EngineFloat64ArrayType.c_type, # Output, phup[ng].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_noise_bivar = utils._EngineState._get_symbol('g13cgf')
multi_noise_bivar.restype = None
multi_noise_bivar.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, xg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, yg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, xyrg[ng].
    utils._EngineFloat64ArrayType.c_type, # Input, xyig[ng].
    utils._BYREF_INT, # Input, ng.
    utils._EngineFloat64ArrayType.c_type, # Input, stats[4].
    utils._BYREF_INT, # Input, l.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Output, er[ng].
    utils._BYREF_FLOAT64, # Output, erlw.
    utils._BYREF_FLOAT64, # Output, erup.
    utils._EngineFloat64ArrayType.c_type, # Output, rf[l].
    utils._BYREF_FLOAT64, # Output, rfse.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_autocorr_part = utils._EngineState._get_symbol('g13dbf')
multi_autocorr_part.restype = None
multi_autocorr_part.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, c0[ldc0, ns].
    utils._EngineFloat64ArrayType.c_type, # Input, c[ldc0, ldc0, nl].
    utils._BYREF_INT, # Input, ldc0.
    utils._BYREF_INT, # Input, ns.
    utils._BYREF_INT, # Input, nl.
    utils._BYREF_INT, # Input, nk.
    utils._EngineFloat64ArrayType.c_type, # Output, p[nk].
    utils._BYREF_FLOAT64, # Output, v0.
    utils._EngineFloat64ArrayType.c_type, # Output, v[nk].
    utils._EngineFloat64ArrayType.c_type, # Output, d[ldc0, ldc0, nk].
    utils._EngineFloat64ArrayType.c_type, # Output, db[ldc0, ns].
    utils._EngineFloat64ArrayType.c_type, # Output, w[ldc0, ldc0, nk].
    utils._EngineFloat64ArrayType.c_type, # Output, wb[ldc0, ldc0, nk].
    utils._BYREF_INT, # Output, nvp.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wa[iwa].
    utils._BYREF_INT, # Input, iwa.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_varma_estimate = utils._EngineState._get_symbol('g13ddf')
multi_varma_estimate.restype = None
multi_varma_estimate.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._BYREF_LOGICAL, # Input, mean.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, par[npar].
    utils._BYREF_INT, # Input, npar.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, qq[kmax, k].
    utils._BYREF_INT, # Input, kmax.
    utils._EngineFloat64ArrayType.c_type, # Input, w[kmax, n].
    utils._EngineLogicalArrayType.c_type, # Input, parhld[npar].
    utils._BYREF_LOGICAL, # Input, exact.
    utils._BYREF_INT, # Input, iprint.
    utils._BYREF_FLOAT64, # Input, cgetol.
    utils._BYREF_INT, # Input, maxcal.
    utils._BYREF_INT, # Input, ishow.
    utils._BYREF_INT, # Output, niter.
    utils._BYREF_FLOAT64, # Output, rlogl.
    utils._EngineFloat64ArrayType.c_type, # Output, v[kmax, n].
    utils._EngineFloat64ArrayType.c_type, # Output, g[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, cm[ldcm, npar].
    utils._BYREF_INT, # Input, ldcm.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_varma_forecast = utils._EngineState._get_symbol('g13djf')
multi_varma_forecast.restype = None
multi_varma_forecast.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, z[kmax, n].
    utils._BYREF_INT, # Input, kmax.
    utils._EngineCharArrayType.c_type, # Input, tr(len=1)[k].
    utils._EngineIntArrayType.c_type, # Input, id[k].
    utils._EngineFloat64ArrayType.c_type, # Input, delta[kmax, *].
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input, par[lpar].
    utils._BYREF_INT, # Input, lpar.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, qq[kmax, k].
    utils._EngineFloat64ArrayType.c_type, # Input, v[kmax, *].
    utils._BYREF_INT, # Input, lmax.
    utils._EngineFloat64ArrayType.c_type, # Output, predz[kmax, lmax].
    utils._EngineFloat64ArrayType.c_type, # Output, sefz[kmax, lmax].
    utils._EngineFloat64ArrayType.c_type, # Output, ref[lref].
    utils._BYREF_INT, # Input, lref.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._EngineIntArrayType.c_type, # Workspace, iwork[liwork].
    utils._BYREF_INT, # Input, liwork.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tr.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
)

multi_varma_update = utils._EngineState._get_symbol('g13dkf')
multi_varma_update.restype = None
multi_varma_update.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, lmax.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input/Output, mlast.
    utils._EngineFloat64ArrayType.c_type, # Input, z[kmax, m].
    utils._BYREF_INT, # Input, kmax.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ref[lref].
    utils._BYREF_INT, # Input, lref.
    utils._EngineFloat64ArrayType.c_type, # Output, v[kmax, m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, predz[kmax, lmax].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, sefz[kmax, lmax].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[k*m].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_diff = utils._EngineState._get_symbol('g13dlf')
multi_diff.restype = None
multi_diff.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, z[kmax, n].
    utils._BYREF_INT, # Input, kmax.
    utils._EngineCharArrayType.c_type, # Input, tr(len=1)[k].
    utils._EngineIntArrayType.c_type, # Input, id[k].
    utils._EngineFloat64ArrayType.c_type, # Input, delta[kmax, *].
    utils._EngineFloat64ArrayType.c_type, # Output, w[kmax, *].
    utils._BYREF_INT, # Output, nd.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[k*n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tr.
)

multi_corrmat_cross = utils._EngineState._get_symbol('g13dmf')
multi_corrmat_cross.restype = None
multi_corrmat_cross.argtypes = (
    utils._BYREF_STRING, # Input, matrix(len=1).
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, w[kmax, n].
    utils._BYREF_INT, # Input, kmax.
    utils._EngineFloat64ArrayType.c_type, # Output, wmean[k].
    utils._EngineFloat64ArrayType.c_type, # Output, r0[kmax, k].
    utils._EngineFloat64ArrayType.c_type, # Output, r[kmax, kmax, m].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of matrix.
)

multi_corrmat_partlag = utils._EngineState._get_symbol('g13dnf')
multi_corrmat_partlag.restype = None
multi_corrmat_partlag.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, kmax.
    utils._EngineFloat64ArrayType.c_type, # Input, r0[kmax, k].
    utils._EngineFloat64ArrayType.c_type, # Input, r[kmax, kmax, m].
    utils._BYREF_INT, # Output, maxlag.
    utils._EngineFloat64ArrayType.c_type, # Output, parlag[kmax, kmax, m].
    utils._EngineFloat64ArrayType.c_type, # Output, x[m].
    utils._EngineFloat64ArrayType.c_type, # Output, pvalue[m].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_regmat_partial = utils._EngineState._get_symbol('g13dpf')
multi_regmat_partial.restype = None
multi_regmat_partial.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, z[kmax, n].
    utils._BYREF_INT, # Input, kmax.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Output, maxlag.
    utils._EngineFloat64ArrayType.c_type, # Output, parlag[kmax, kmax, m].
    utils._EngineFloat64ArrayType.c_type, # Output, se[kmax, kmax, m].
    utils._EngineFloat64ArrayType.c_type, # Output, qq[kmax, kmax, m].
    utils._EngineFloat64ArrayType.c_type, # Output, x[m].
    utils._EngineFloat64ArrayType.c_type, # Output, pvalue[m].
    utils._EngineFloat64ArrayType.c_type, # Output, loglhd[m].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._EngineIntArrayType.c_type, # Workspace, iwork[k*m].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_varma_diag = utils._EngineState._get_symbol('g13dsf')
multi_varma_diag.restype = None
multi_varma_diag.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, v[kmax, n].
    utils._BYREF_INT, # Input, kmax.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, par[(ip+iq)*k*k].
    utils._EngineLogicalArrayType.c_type, # Input, parhld[(ip+iq)*k*k].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, qq[kmax, k].
    utils._BYREF_INT, # Input, ishow.
    utils._EngineFloat64ArrayType.c_type, # Output, r0[kmax, k].
    utils._EngineFloat64ArrayType.c_type, # Output, r[kmax, kmax, m].
    utils._EngineFloat64ArrayType.c_type, # Output, rcm[ldrcm, m*k*k].
    utils._BYREF_INT, # Input, ldrcm.
    utils._BYREF_FLOAT64, # Output, chi.
    utils._BYREF_INT, # Output, idf.
    utils._BYREF_FLOAT64, # Output, siglev.
    utils._EngineIntArrayType.c_type, # Workspace, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_arma_roots = utils._EngineState._get_symbol('g13dxf')
uni_arma_roots.restype = None
uni_arma_roots.argtypes = (
    utils._BYREF_INT, # Input, k.
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, par[ip*k*k].
    utils._EngineFloat64ArrayType.c_type, # Output, rr[k*ip].
    utils._EngineFloat64ArrayType.c_type, # Output, ri[k*ip].
    utils._EngineFloat64ArrayType.c_type, # Output, rmod[k*ip].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[k*k*ip*ip].
    utils._EngineIntArrayType.c_type, # Workspace, iwork[k*ip].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_kalman_sqrt_var = utils._EngineState._get_symbol('g13eaf')
multi_kalman_sqrt_var.restype = None
multi_kalman_sqrt_var.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, l.
    utils._EngineFloat64ArrayType.c_type, # Input, a[lds, n].
    utils._BYREF_INT, # Input, lds.
    utils._EngineFloat64ArrayType.c_type, # Input, b[lds, l].
    utils._BYREF_LOGICAL, # Input, stq.
    utils._EngineFloat64ArrayType.c_type, # Input, q[ldq, *].
    utils._BYREF_INT, # Input, ldq.
    utils._EngineFloat64ArrayType.c_type, # Input, c[ldm, n].
    utils._BYREF_INT, # Input, ldm.
    utils._EngineFloat64ArrayType.c_type, # Input, r[ldm, m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, s[lds, n].
    utils._EngineFloat64ArrayType.c_type, # Output, k[lds, m].
    utils._EngineFloat64ArrayType.c_type, # Output, h[ldm, m].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineIntArrayType.c_type, # Workspace, iwk[m].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[(n+m)*(n+m+l)].
    utils._BYREF_INT, # Input/Output, ifail.
)

multi_kalman_sqrt_invar = utils._EngineState._get_symbol('g13ebf')
multi_kalman_sqrt_invar.restype = None
multi_kalman_sqrt_invar.argtypes = (
    utils._BYREF_STRING, # Input, transf(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, l.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, a[lds, n].
    utils._BYREF_INT, # Input, lds.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, b[lds, l].
    utils._BYREF_LOGICAL, # Input, stq.
    utils._EngineFloat64ArrayType.c_type, # Input, q[ldq, *].
    utils._BYREF_INT, # Input, ldq.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, c[ldm, n].
    utils._BYREF_INT, # Input, ldm.
    utils._EngineFloat64ArrayType.c_type, # Input, r[ldm, m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, s[lds, n].
    utils._EngineFloat64ArrayType.c_type, # Output, k[lds, m].
    utils._EngineFloat64ArrayType.c_type, # Output, h[ldm, m].
    utils._EngineFloat64ArrayType.c_type, # Output, u[lds, *].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineIntArrayType.c_type, # Workspace, iwk[m].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[(n+m)*(n+m+l)].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of transf.
)

kalman_unscented_state_revcom = utils._EngineState._get_symbol('g13ejf')
kalman_unscented_state_revcom.restype = None
kalman_unscented_state_revcom.argtypes = (
    utils._BYREF_INT, # Input/Output, irevcm.
    utils._BYREF_INT, # Input, mx.
    utils._BYREF_INT, # Input, my.
    utils._EngineFloat64ArrayType.c_type, # Input, y[my].
    utils._EngineFloat64ArrayType.c_type, # Input, lx[ldlx, *].
    utils._BYREF_INT, # Input, ldlx.
    utils._EngineFloat64ArrayType.c_type, # Input, ly[ldly, *].
    utils._BYREF_INT, # Input, ldly.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[mx].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, st[ldst, *].
    utils._BYREF_INT, # Input, ldst.
    utils._BYREF_INT, # Input/Output, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xt[ldxt, *].
    utils._BYREF_INT, # Input, ldxt.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, fxt[ldfxt, *].
    utils._BYREF_INT, # Input, ldfxt.
    utils._EngineFloat64ArrayType.c_type, # Input, ropt[lropt].
    utils._BYREF_INT, # Input, lropt.
    utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
    utils._BYREF_INT, # Input, licomm.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._BYREF_INT, # Input/Output, ifail.
)

kalman_unscented_state_f = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, mx.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, xt[mx, n].
    utils._EngineFloat64ArrayType.c_type, # Output, fxt[mx, n].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, info.
)
kalman_unscented_state_h = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, mx.
    utils._BYREF_INT, # Input, my.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, yt[mx, n].
    utils._EngineFloat64ArrayType.c_type, # Output, hyt[my, n].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, info.
)
kalman_unscented_state = utils._EngineState._get_symbol('g13ekf')
kalman_unscented_state.restype = None
kalman_unscented_state.argtypes = (
    utils._BYREF_INT, # Input, mx.
    utils._BYREF_INT, # Input, my.
    utils._EngineFloat64ArrayType.c_type, # Input, y[my].
    utils._EngineFloat64ArrayType.c_type, # Input, lx[mx, mx].
    utils._EngineFloat64ArrayType.c_type, # Input, ly[my, my].
    kalman_unscented_state_f,
    kalman_unscented_state_h,
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[mx].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, st[mx, mx].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_garch_asym1_estim = utils._EngineState._get_symbol('g13faf')
uni_garch_asym1_estim.restype = None
uni_garch_asym1_estim.argtypes = (
    utils._BYREF_STRING, # Input, dist(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input, yt[num].
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._BYREF_INT, # Input, nreg.
    utils._BYREF_INT, # Input, mn.
    utils._BYREF_INT, # Input, isym.
    utils._BYREF_INT, # Input, npar.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, se[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, sc[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, covr[ldcovr, npar].
    utils._BYREF_INT, # Input, ldcovr.
    utils._BYREF_FLOAT64, # Input/Output, pht.
    utils._EngineFloat64ArrayType.c_type, # Output, et[num].
    utils._EngineFloat64ArrayType.c_type, # Output, ht[num].
    utils._BYREF_FLOAT64, # Output, lgf.
    utils._EngineLogicalArrayType.c_type, # Input, copts[2].
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of dist.
)

uni_garch_asym1_forecast = utils._EngineState._get_symbol('g13fbf')
uni_garch_asym1_forecast.restype = None
uni_garch_asym1_forecast.argtypes = (
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, nt.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._EngineFloat64ArrayType.c_type, # Input, theta[iq+ip+1].
    utils._BYREF_FLOAT64, # Input, gamma.
    utils._EngineFloat64ArrayType.c_type, # Output, fht[nt].
    utils._EngineFloat64ArrayType.c_type, # Input, ht[num].
    utils._EngineFloat64ArrayType.c_type, # Input, et[num].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_garch_asym2_estim = utils._EngineState._get_symbol('g13fcf')
uni_garch_asym2_estim.restype = None
uni_garch_asym2_estim.argtypes = (
    utils._BYREF_STRING, # Input, dist(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input, yt[num].
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._BYREF_INT, # Input, nreg.
    utils._BYREF_INT, # Input, mn.
    utils._BYREF_INT, # Input, npar.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, se[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, sc[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, covr[ldcovr, npar].
    utils._BYREF_INT, # Input, ldcovr.
    utils._BYREF_FLOAT64, # Input/Output, pht.
    utils._EngineFloat64ArrayType.c_type, # Output, et[num].
    utils._EngineFloat64ArrayType.c_type, # Output, ht[num].
    utils._BYREF_FLOAT64, # Output, lgf.
    utils._EngineLogicalArrayType.c_type, # Input, copts[2].
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of dist.
)

uni_garch_asym2_forecast = utils._EngineState._get_symbol('g13fdf')
uni_garch_asym2_forecast.restype = None
uni_garch_asym2_forecast.argtypes = (
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, nt.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._EngineFloat64ArrayType.c_type, # Input, theta[iq+ip+1].
    utils._BYREF_FLOAT64, # Input, gamma.
    utils._EngineFloat64ArrayType.c_type, # Output, fht[nt].
    utils._EngineFloat64ArrayType.c_type, # Input, ht[num].
    utils._EngineFloat64ArrayType.c_type, # Input, et[num].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_garch_gjr_estim = utils._EngineState._get_symbol('g13fef')
uni_garch_gjr_estim.restype = None
uni_garch_gjr_estim.argtypes = (
    utils._BYREF_STRING, # Input, dist(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input, yt[num].
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._BYREF_INT, # Input, nreg.
    utils._BYREF_INT, # Input, mn.
    utils._BYREF_INT, # Input, npar.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, se[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, sc[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, covr[ldcovr, npar].
    utils._BYREF_INT, # Input, ldcovr.
    utils._BYREF_FLOAT64, # Input/Output, pht.
    utils._EngineFloat64ArrayType.c_type, # Output, et[num].
    utils._EngineFloat64ArrayType.c_type, # Output, ht[num].
    utils._BYREF_FLOAT64, # Output, lgf.
    utils._EngineLogicalArrayType.c_type, # Input, copts[2].
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of dist.
)

uni_garch_gjr_forecast = utils._EngineState._get_symbol('g13fff')
uni_garch_gjr_forecast.restype = None
uni_garch_gjr_forecast.argtypes = (
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, nt.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._EngineFloat64ArrayType.c_type, # Input, theta[iq+ip+1].
    utils._BYREF_FLOAT64, # Input, gamma.
    utils._EngineFloat64ArrayType.c_type, # Output, fht[nt].
    utils._EngineFloat64ArrayType.c_type, # Input, ht[num].
    utils._EngineFloat64ArrayType.c_type, # Input, et[num].
    utils._BYREF_INT, # Input/Output, ifail.
)

uni_garch_exp_estim = utils._EngineState._get_symbol('g13fgf')
uni_garch_exp_estim.restype = None
uni_garch_exp_estim.argtypes = (
    utils._BYREF_STRING, # Input, dist(len=1).
    utils._EngineFloat64ArrayType.c_type, # Input, yt[num].
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._BYREF_INT, # Input, nreg.
    utils._BYREF_INT, # Input, mn.
    utils._BYREF_INT, # Input, npar.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, se[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, sc[npar].
    utils._EngineFloat64ArrayType.c_type, # Output, covr[ldcovr, npar].
    utils._BYREF_INT, # Input, ldcovr.
    utils._BYREF_FLOAT64, # Input/Output, pht.
    utils._EngineFloat64ArrayType.c_type, # Output, et[num].
    utils._EngineFloat64ArrayType.c_type, # Output, ht[num].
    utils._BYREF_FLOAT64, # Output, lgf.
    utils._BYREF_LOGICAL, # Input, copts.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[lwork].
    utils._BYREF_INT, # Input, lwork.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of dist.
)

uni_garch_exp_forecast = utils._EngineState._get_symbol('g13fhf')
uni_garch_exp_forecast.restype = None
uni_garch_exp_forecast.argtypes = (
    utils._BYREF_INT, # Input, num.
    utils._BYREF_INT, # Input, nt.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iq.
    utils._EngineFloat64ArrayType.c_type, # Input, theta[2*iq+ip+1].
    utils._EngineFloat64ArrayType.c_type, # Output, fht[nt].
    utils._EngineFloat64ArrayType.c_type, # Input, ht[num].
    utils._EngineFloat64ArrayType.c_type, # Input, et[num].
    utils._BYREF_INT, # Input/Output, ifail.
)

inhom_iema = utils._EngineState._get_symbol('g13mef')
inhom_iema.restype = None
inhom_iema.argtypes = (
    utils._BYREF_INT, # Input, nb.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, iema[nb].
    utils._EngineFloat64ArrayType.c_type, # Input, t[nb].
    utils._BYREF_FLOAT64, # Input, tau.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, sinit[m+2].
    utils._EngineIntArrayType.c_type, # Input, inter[2].
    utils._BYREF_INT, # Input/Output, pn.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._BYREF_INT, # Input/Output, ifail.
)

inhom_iema_all = utils._EngineState._get_symbol('g13mff')
inhom_iema_all.restype = None
inhom_iema_all.argtypes = (
    utils._BYREF_INT, # Input, sorder.
    utils._BYREF_INT, # Input, nb.
    utils._EngineFloat64ArrayType.c_type, # Input, z[nb].
    utils._EngineFloat64ArrayType.c_type, # Output, iema[ldiema, *].
    utils._BYREF_INT, # Input, ldiema.
    utils._EngineFloat64ArrayType.c_type, # Input, t[nb].
    utils._BYREF_FLOAT64, # Input, tau.
    utils._BYREF_INT, # Input, m1.
    utils._BYREF_INT, # Input, m2.
    utils._EngineFloat64ArrayType.c_type, # Input, sinit[m2+2].
    utils._EngineIntArrayType.c_type, # Input, inter[2].
    utils._BYREF_INT, # Input, ftype.
    utils._BYREF_FLOAT64, # Input/Output, p.
    utils._EngineFloat64ArrayType.c_type, # Input, x[*].
    utils._BYREF_INT, # Input/Output, pn.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._BYREF_INT, # Input/Output, ifail.
)

inhom_ma = utils._EngineState._get_symbol('g13mgf')
inhom_ma.restype = None
inhom_ma.argtypes = (
    utils._BYREF_INT, # Input, nb.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ma[nb].
    utils._EngineFloat64ArrayType.c_type, # Input, t[nb].
    utils._BYREF_FLOAT64, # Input, tau.
    utils._BYREF_INT, # Input, m1.
    utils._BYREF_INT, # Input, m2.
    utils._EngineFloat64ArrayType.c_type, # Input, sinit[*].
    utils._EngineIntArrayType.c_type, # Input, inter[2].
    utils._BYREF_INT, # Input, ftype.
    utils._BYREF_FLOAT64, # Input/Output, p.
    utils._BYREF_INT, # Input/Output, pn.
    utils._EngineFloat64ArrayType.c_type, # Output, wma[nb].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._BYREF_INT, # Input/Output, ifail.
)

cp_pelt = utils._EngineState._get_symbol('g13naf')
cp_pelt.restype = None
cp_pelt.argtypes = (
    utils._BYREF_INT, # Input, ctype.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_FLOAT64, # Input, beta.
    utils._BYREF_INT, # Input, minss.
    utils._BYREF_INT, # Input, iparam.
    utils._EngineFloat64ArrayType.c_type, # Input, param[1].
    utils._BYREF_INT, # Output, ntau.
    utils._EngineIntArrayType.c_type, # Output, tau[n].
    utils._EngineFloat64ArrayType.c_type, # Output, sparam[2*n+2].
    utils._BYREF_INT, # Input/Output, ifail.
)

cp_pelt_user_costfn = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ts.
    utils._BYREF_INT, # Input, nr.
    utils._EngineIntArrayType.c_type, # Input, r[nr].
    utils._EngineFloat64ArrayType.c_type, # Output, c[nr].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[*].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, info.
)
cp_pelt_user = utils._EngineState._get_symbol('g13nbf')
cp_pelt_user.restype = None
cp_pelt_user.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, beta.
    utils._BYREF_INT, # Input, minss.
    utils._BYREF_FLOAT64, # Input, k.
    cp_pelt_user_costfn,
    utils._BYREF_INT, # Output, ntau.
    utils._EngineIntArrayType.c_type, # Output, tau[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[*].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

cp_binary = utils._EngineState._get_symbol('g13ndf')
cp_binary.restype = None
cp_binary.argtypes = (
    utils._BYREF_INT, # Input, ctype.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_FLOAT64, # Input, beta.
    utils._BYREF_INT, # Input, minss.
    utils._BYREF_INT, # Input, iparam.
    utils._EngineFloat64ArrayType.c_type, # Input, param[1].
    utils._BYREF_INT, # Input, mdepth.
    utils._BYREF_INT, # Output, ntau.
    utils._EngineIntArrayType.c_type, # Output, tau[*].
    utils._EngineFloat64ArrayType.c_type, # Output, sparam[2*n].
    utils._BYREF_INT, # Input/Output, ifail.
)

cp_binary_user_chgpfn = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, side.
    utils._BYREF_INT, # Input, u.
    utils._BYREF_INT, # Input, w.
    utils._BYREF_INT, # Input, minss.
    utils._BYREF_INT, # Output, v.
    utils._EngineFloat64ArrayType.c_type, # Output, cost[3].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[*].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, info.
)
cp_binary_user = utils._EngineState._get_symbol('g13nef')
cp_binary_user.restype = None
cp_binary_user.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, beta.
    utils._BYREF_INT, # Input, minss.
    utils._BYREF_INT, # Input, mdepth.
    cp_binary_user_chgpfn,
    utils._BYREF_INT, # Output, ntau.
    utils._EngineIntArrayType.c_type, # Output, tau[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[*].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)
