"""
Deprecated wrappers for the NAG C Library (Mark 26.0) f11 Chapter.

f11 - Large Scale Linear Systems.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/F11/f11conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER

from .util import get_library

from .util import Nag_SparseNsym_Method
from .util import Nag_SparseNsym_PrecType
from .util import Nag_NormType
from .util import Nag_SparseNsym_Weight
from .util import Integer
from .util import NagError
from .util import Complex
from .util import Nag_SparseNsym_Piv
from .util import Nag_SparseNsym_Fact
from .util import Nag_TransType
from .util import Nag_SparseNsym_CheckData
from .util import Nag_Sparse_Comm
from .util import Nag_SparseNsym_Store
from .util import Nag_InitializeA
from .util import Nag_SparseSym_Method
from .util import Nag_SparseSym_PrecType
from .util import Nag_SparseSym_Bisection
from .util import Nag_SparseSym_Weight
from .util import Nag_SparseSym_Fact
from .util import Nag_SparseSym_Piv
from .util import Nag_SparseSym_CheckData
from .util import Nag_ColumnPermutationType
from .util import Nag_OrderType
from .util import Nag_SparseNsym_Dups
from .util import Nag_SparseNsym_Zeros
from .util import Nag_SparseSym_Dups
from .util import Nag_SparseSym_Zeros
from .util import Nag_Boolean
_library_cl = get_library()

f11bdc_ctypes = _library_cl.f11bdc
f11bdc_ctypes.restype = None
f11bdc_ctypes.argtypes = [Nag_SparseNsym_Method, Nag_SparseNsym_PrecType, Nag_NormType, Nag_SparseNsym_Weight, Integer, Integer, Integer, c_double, Integer, c_double, c_double, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(NagError)]


def f11bdc(method, precon, norm, weight, iterm, n, m, tol, maxitn, anorm, sigmax, monit, lwreq, work, lwork, fail):
    """
    Real sparse nonsymmetric linear systems, setup for f11bec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11bdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lwreq, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        lwreq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lwreq = nag_integer_type_check_and_cast('f11bdc', lwreq, 'lwreq')
    _work = nag_double_type_check_and_cast('f11bdc', work, 'work')
    f11bdc_ctypes(method, precon, norm, weight, iterm, n, m, tol, maxitn, anorm, sigmax, monit, _lwreq, _work, lwork, fail)

f11bec_ctypes = _library_cl.f11bec
f11bec_ctypes.restype = None
f11bec_ctypes.argtypes = [POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f11bec(irevcm, u, v, wgt, work, lwork, fail):
    """
    Real sparse nonsymmetric linear systems, preconditioned RGMRES, CGS, Bi-
    CGSTAB or TFQMR method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, u, v, wgt, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        u, v, wgt, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f11bec', irevcm, 'irevcm')
    _u = nag_double_type_check_and_cast('f11bec', u, 'u')
    _v = nag_double_type_check_and_cast('f11bec', v, 'v')
    _wgt = nag_double_type_check_and_cast('f11bec', wgt, 'wgt')
    _work = nag_double_type_check_and_cast('f11bec', work, 'work')
    f11bec_ctypes(_irevcm, _u, _v, _wgt, _work, lwork, fail)

f11bfc_ctypes = _library_cl.f11bfc
f11bfc_ctypes.restype = None
f11bfc_ctypes.argtypes = [POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f11bfc(itn, stplhs, stprhs, anorm, sigmax, work, lwork, fail):
    """
    Real sparse nonsymmetric linear systems, diagnostic for f11bec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11bfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        itn, stplhs, stprhs, anorm, sigmax, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        stplhs, stprhs, anorm, sigmax, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        itn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _itn = nag_integer_type_check_and_cast('f11bfc', itn, 'itn')
    _stplhs = nag_double_type_check_and_cast('f11bfc', stplhs, 'stplhs')
    _stprhs = nag_double_type_check_and_cast('f11bfc', stprhs, 'stprhs')
    _anorm = nag_double_type_check_and_cast('f11bfc', anorm, 'anorm')
    _sigmax = nag_double_type_check_and_cast('f11bfc', sigmax, 'sigmax')
    _work = nag_double_type_check_and_cast('f11bfc', work, 'work')
    f11bfc_ctypes(_itn, _stplhs, _stprhs, _anorm, _sigmax, _work, lwork, fail)

f11brc_ctypes = _library_cl.f11brc
f11brc_ctypes.restype = None
f11brc_ctypes.argtypes = [Nag_SparseNsym_Method, Nag_SparseNsym_PrecType, Nag_NormType, Nag_SparseNsym_Weight, Integer, Integer, Integer, c_double, Integer, c_double, c_double, Integer, POINTER(Integer), POINTER(Complex), Integer, POINTER(NagError)]


def f11brc(method, precon, norm, weight, iterm, n, m, tol, maxitn, anorm, sigmax, monit, lwreq, work, lwork, fail):
    """
    Complex sparse non-Hermitian linear systems, setup for f11bsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11brc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lwreq, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        lwreq
    TypeError will be raised when the following arguments are not of data type
    Complex:
        work
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lwreq = nag_integer_type_check_and_cast('f11brc', lwreq, 'lwreq')
    _work = nag_complex_type_check_and_cast('f11brc', work, 'work')
    f11brc_ctypes(method, precon, norm, weight, iterm, n, m, tol, maxitn, anorm, sigmax, monit, _lwreq, _work, lwork, fail)

f11bsc_ctypes = _library_cl.f11bsc
f11bsc_ctypes.restype = None
f11bsc_ctypes.argtypes = [POINTER(Integer), POINTER(Complex), POINTER(Complex), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f11bsc(irevcm, u, v, wgt, work, lwork, fail):
    """
    Complex sparse non-Hermitian linear systems, preconditioned RGMRES, CGS,
    Bi-CGSTAB or TFQMR method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11bsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, u, v, wgt, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        wgt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm
    TypeError will be raised when the following arguments are not of data type
    Complex:
        u, v, work
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f11bsc', irevcm, 'irevcm')
    _u = nag_complex_type_check_and_cast('f11bsc', u, 'u')
    _v = nag_complex_type_check_and_cast('f11bsc', v, 'v')
    _wgt = nag_double_type_check_and_cast('f11bsc', wgt, 'wgt')
    _work = nag_complex_type_check_and_cast('f11bsc', work, 'work')
    f11bsc_ctypes(_irevcm, _u, _v, _wgt, _work, lwork, fail)

f11btc_ctypes = _library_cl.f11btc
f11btc_ctypes.restype = None
f11btc_ctypes.argtypes = [POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f11btc(itn, stplhs, stprhs, anorm, sigmax, work, lwork, fail):
    """
    Complex sparse non-Hermitian linear systems, diagnostic for f11bsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11btc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        itn, stplhs, stprhs, anorm, sigmax, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        stplhs, stprhs, anorm, sigmax
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        itn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        work
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _itn = nag_integer_type_check_and_cast('f11btc', itn, 'itn')
    _stplhs = nag_double_type_check_and_cast('f11btc', stplhs, 'stplhs')
    _stprhs = nag_double_type_check_and_cast('f11btc', stprhs, 'stprhs')
    _anorm = nag_double_type_check_and_cast('f11btc', anorm, 'anorm')
    _sigmax = nag_double_type_check_and_cast('f11btc', sigmax, 'sigmax')
    _work = nag_complex_type_check_and_cast('f11btc', work, 'work')
    f11btc_ctypes(_itn, _stplhs, _stprhs, _anorm, _sigmax, _work, lwork, fail)

f11dac_ctypes = _library_cl.f11dac
f11dac_ctypes.restype = None
f11dac_ctypes.argtypes = [Integer, Integer, POINTER(POINTER(c_double)), POINTER(Integer), POINTER(POINTER(Integer)), POINTER(POINTER(Integer)), Integer, c_double, Nag_SparseNsym_Piv, Nag_SparseNsym_Fact, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f11dac(n, nnz, a, la, irow, icol, lfill, dtol, pstrat, milu, ipivp, ipivq, istr, idiag, nnzc, npivm, fail):
    """
    Incomplete LU factorization (nonsymmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, la, irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        la, irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11dac', a, 'a')
    _la = nag_integer_type_check_and_cast('f11dac', la, 'la')
    _irow = nag_integer_type_check_and_cast('f11dac', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dac', icol, 'icol')
    _ipivp = nag_integer_type_check_and_cast('f11dac', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dac', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dac', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dac', idiag, 'idiag')
    _nnzc = nag_integer_type_check_and_cast('f11dac', nnzc, 'nnzc')
    _npivm = nag_integer_type_check_and_cast('f11dac', npivm, 'npivm')
    f11dac_ctypes(n, nnz, _a, _la, _irow, _icol, lfill, dtol, pstrat, milu, _ipivp, _ipivq, _istr, _idiag, _nnzc, _npivm, fail)

f11dbc_ctypes = _library_cl.f11dbc
f11dbc_ctypes.restype = None
f11dbc_ctypes.argtypes = [Nag_TransType, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_CheckData, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11dbc(trans, n, a, la, irow, icol, ipivp, ipivq, istr, idiag, check, y, x, fail):
    """
    Solution of linear system involving incomplete LU preconditioning matrix
    generated by f11dac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipivp, ipivq, istr, idiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipivp, ipivq, istr, idiag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11dbc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dbc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dbc', icol, 'icol')
    _ipivp = nag_integer_type_check_and_cast('f11dbc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dbc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dbc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dbc', idiag, 'idiag')
    _y = nag_double_type_check_and_cast('f11dbc', y, 'y')
    _x = nag_double_type_check_and_cast('f11dbc', x, 'x')
    f11dbc_ctypes(trans, n, _a, la, _irow, _icol, _ipivp, _ipivq, _istr, _idiag, check, _y, _x, fail)

f11dcc_ctypes = _library_cl.f11dcc
f11dcc_ctypes.restype = None
f11dcc_ctypes.argtypes = [Nag_SparseNsym_Method, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Nag_Sparse_Comm), POINTER(NagError)]


def f11dcc(method, n, nnz, a, la, irow, icol, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x, rnorm, itn, comm, fail):
    """
    Solver with incomplete LU preconditioning (nonsymmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipivp, ipivq, istr, idiag, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipivp, ipivq, istr, idiag, itn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11dcc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dcc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dcc', icol, 'icol')
    _ipivp = nag_integer_type_check_and_cast('f11dcc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dcc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dcc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dcc', idiag, 'idiag')
    _b = nag_double_type_check_and_cast('f11dcc', b, 'b')
    _x = nag_double_type_check_and_cast('f11dcc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11dcc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11dcc', itn, 'itn')
    f11dcc_ctypes(method, n, nnz, _a, la, _irow, _icol, _ipivp, _ipivq, _istr, _idiag, _b, m, tol, maxitn, _x, _rnorm, _itn, comm, fail)

f11ddc_ctypes = _library_cl.f11ddc
f11ddc_ctypes.restype = None
f11ddc_ctypes.argtypes = [Nag_TransType, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), c_double, Nag_SparseNsym_CheckData, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11ddc(trans, n, nnz, a, irow, icol, rdiag, omega, check, y, x, fail):
    """
    Solution of linear system involving preconditioning matrix generated by
    applying SSOR to real sparse nonsymmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11ddc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, rdiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, rdiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11ddc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11ddc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11ddc', icol, 'icol')
    _rdiag = nag_double_type_check_and_cast('f11ddc', rdiag, 'rdiag')
    _y = nag_double_type_check_and_cast('f11ddc', y, 'y')
    _x = nag_double_type_check_and_cast('f11ddc', x, 'x')
    f11ddc_ctypes(trans, n, nnz, _a, _irow, _icol, _rdiag, omega, check, _y, _x, fail)

f11dec_ctypes = _library_cl.f11dec
f11dec_ctypes.restype = None
f11dec_ctypes.argtypes = [Nag_SparseNsym_Method, Nag_SparseNsym_PrecType, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), c_double, POINTER(c_double), Integer, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Nag_Sparse_Comm), POINTER(NagError)]


def f11dec(method, precon, n, nnz, a, irow, icol, omega, b, m, tol, maxitn, x, rnorm, itn, comm, fail):
    """
    Solver with no Jacobi/SSOR preconditioning (nonsymmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, itn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11dec', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dec', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dec', icol, 'icol')
    _b = nag_double_type_check_and_cast('f11dec', b, 'b')
    _x = nag_double_type_check_and_cast('f11dec', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11dec', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11dec', itn, 'itn')
    f11dec_ctypes(method, precon, n, nnz, _a, _irow, _icol, omega, _b, m, tol, maxitn, _x, _rnorm, _itn, comm, fail)

f11dfc_ctypes = _library_cl.f11dfc
f11dfc_ctypes.restype = None
f11dfc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(Nag_SparseNsym_Piv), POINTER(Nag_SparseNsym_Fact), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f11dfc(n, nnz, a, la, irow, icol, nb, istb, indb, lindb, lfill, dtol, pstrat, milu, ipivp, ipivq, istr, idiag, nnzc, npivm, fail):
    """
    Real sparse nonsymmetric linear system, incomplete LU factorization of
    local or overlapping diagonal blocks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, istb, indb, lfill, dtol, ipivp, ipivq, istr, idiag, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, dtol
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, istb, indb, lfill, ipivp, ipivq, istr, idiag, nnzc, npivm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11dfc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dfc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dfc', icol, 'icol')
    _istb = nag_integer_type_check_and_cast('f11dfc', istb, 'istb')
    _indb = nag_integer_type_check_and_cast('f11dfc', indb, 'indb')
    _lfill = nag_integer_type_check_and_cast('f11dfc', lfill, 'lfill')
    _dtol = nag_double_type_check_and_cast('f11dfc', dtol, 'dtol')
    _ipivp = nag_integer_type_check_and_cast('f11dfc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dfc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dfc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dfc', idiag, 'idiag')
    _nnzc = nag_integer_type_check_and_cast('f11dfc', nnzc, 'nnzc')
    _npivm = nag_integer_type_check_and_cast('f11dfc', npivm, 'npivm')
    f11dfc_ctypes(n, nnz, _a, la, _irow, _icol, nb, _istb, _indb, lindb, _lfill, _dtol, pstrat, milu, _ipivp, _ipivq, _istr, _idiag, _nnzc, _npivm, fail)

f11dgc_ctypes = _library_cl.f11dgc
f11dgc_ctypes.restype = None
f11dgc_ctypes.argtypes = [Nag_SparseNsym_Method, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f11dgc(method, n, nnz, a, la, irow, icol, nb, istb, indb, lindb, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x, rnorm, itn, fail):
    """
    Solution of real sparse nonsymmetric linear system, RGMRES, CGS, Bi-
    CGSTAB or TFQMR method, incomplete LU block diagonal preconditioner
    computed by f11dfc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, istb, indb, ipivp, ipivq, istr, idiag, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, istb, indb, ipivp, ipivq, istr, idiag, itn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11dgc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dgc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dgc', icol, 'icol')
    _istb = nag_integer_type_check_and_cast('f11dgc', istb, 'istb')
    _indb = nag_integer_type_check_and_cast('f11dgc', indb, 'indb')
    _ipivp = nag_integer_type_check_and_cast('f11dgc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dgc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dgc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dgc', idiag, 'idiag')
    _b = nag_double_type_check_and_cast('f11dgc', b, 'b')
    _x = nag_double_type_check_and_cast('f11dgc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11dgc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11dgc', itn, 'itn')
    f11dgc_ctypes(method, n, nnz, _a, la, _irow, _icol, nb, _istb, _indb, lindb, _ipivp, _ipivq, _istr, _idiag, _b, m, tol, maxitn, _x, _rnorm, _itn, fail)

f11dkc_ctypes = _library_cl.f11dkc
f11dkc_ctypes.restype = None
f11dkc_ctypes.argtypes = [Nag_SparseNsym_Store, Nag_TransType, Nag_InitializeA, Integer, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_CheckData, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11dkc(store, trans, init, niter, n, nnz, a, irow, icol, check, b, x, diag, fail):
    """
    Real, sparse, symmetric or nonsymmetric, linear systems, line Jacobi
    preconditioner.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, b, x, diag
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, diag
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11dkc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dkc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dkc', icol, 'icol')
    _b = nag_double_type_check_and_cast('f11dkc', b, 'b')
    _x = nag_double_type_check_and_cast('f11dkc', x, 'x')
    _diag = nag_double_type_check_and_cast('f11dkc', diag, 'diag')
    f11dkc_ctypes(store, trans, init, niter, n, nnz, _a, _irow, _icol, check, _b, _x, _diag, fail)

f11dnc_ctypes = _library_cl.f11dnc
f11dnc_ctypes.restype = None
f11dnc_ctypes.argtypes = [Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), Integer, c_double, Nag_SparseNsym_Piv, Nag_SparseNsym_Fact, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f11dnc(n, nnz, a, la, irow, icol, lfill, dtol, pstrat, milu, ipivp, ipivq, istr, idiag, nnzc, npivm, fail):
    """
    Complex sparse non-Hermitian linear systems, incomplete LU
    factorization.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipivp, ipivq, istr, idiag, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11dnc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dnc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dnc', icol, 'icol')
    _ipivp = nag_integer_type_check_and_cast('f11dnc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dnc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dnc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dnc', idiag, 'idiag')
    _nnzc = nag_integer_type_check_and_cast('f11dnc', nnzc, 'nnzc')
    _npivm = nag_integer_type_check_and_cast('f11dnc', npivm, 'npivm')
    f11dnc_ctypes(n, nnz, _a, la, _irow, _icol, lfill, dtol, pstrat, milu, _ipivp, _ipivq, _istr, _idiag, _nnzc, _npivm, fail)

f11dpc_ctypes = _library_cl.f11dpc
f11dpc_ctypes.restype = None
f11dpc_ctypes.argtypes = [Nag_TransType, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_CheckData, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f11dpc(trans, n, a, la, irow, icol, ipivp, ipivq, istr, idiag, check, y, x, fail):
    """
    Solution of complex linear system involving incomplete LU
    preconditioning matrix generated by f11dnc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipivp, ipivq, istr, idiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipivp, ipivq, istr, idiag
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, y, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11dpc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dpc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dpc', icol, 'icol')
    _ipivp = nag_integer_type_check_and_cast('f11dpc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dpc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dpc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dpc', idiag, 'idiag')
    _y = nag_complex_type_check_and_cast('f11dpc', y, 'y')
    _x = nag_complex_type_check_and_cast('f11dpc', x, 'x')
    f11dpc_ctypes(trans, n, _a, la, _irow, _icol, _ipivp, _ipivq, _istr, _idiag, check, _y, _x, fail)

f11dqc_ctypes = _library_cl.f11dqc
f11dqc_ctypes.restype = None
f11dqc_ctypes.argtypes = [Nag_SparseNsym_Method, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Complex), Integer, c_double, Integer, POINTER(Complex), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f11dqc(method, n, nnz, a, la, irow, icol, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x, rnorm, itn, fail):
    """
    Solution of complex sparse non-Hermitian linear system, RGMRES, CGS, Bi-
    CGSTAB or TFQMR method, preconditioner computed by f11dnc (Black Box).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipivp, ipivq, istr, idiag, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipivp, ipivq, istr, idiag, itn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11dqc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dqc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dqc', icol, 'icol')
    _ipivp = nag_integer_type_check_and_cast('f11dqc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dqc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dqc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dqc', idiag, 'idiag')
    _b = nag_complex_type_check_and_cast('f11dqc', b, 'b')
    _x = nag_complex_type_check_and_cast('f11dqc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11dqc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11dqc', itn, 'itn')
    f11dqc_ctypes(method, n, nnz, _a, la, _irow, _icol, _ipivp, _ipivq, _istr, _idiag, _b, m, tol, maxitn, _x, _rnorm, _itn, fail)

f11drc_ctypes = _library_cl.f11drc
f11drc_ctypes.restype = None
f11drc_ctypes.argtypes = [Nag_TransType, Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Integer), POINTER(Complex), c_double, Nag_SparseNsym_CheckData, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f11drc(trans, n, nnz, a, irow, icol, rdiag, omega, check, y, x, fail):
    """
    Solution of linear system involving preconditioning matrix generated by
    applying SSOR to complex sparse non-Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11drc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, rdiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, rdiag, y, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11drc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11drc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11drc', icol, 'icol')
    _rdiag = nag_complex_type_check_and_cast('f11drc', rdiag, 'rdiag')
    _y = nag_complex_type_check_and_cast('f11drc', y, 'y')
    _x = nag_complex_type_check_and_cast('f11drc', x, 'x')
    f11drc_ctypes(trans, n, nnz, _a, _irow, _icol, _rdiag, omega, check, _y, _x, fail)

f11dsc_ctypes = _library_cl.f11dsc
f11dsc_ctypes.restype = None
f11dsc_ctypes.argtypes = [Nag_SparseNsym_Method, Nag_SparseNsym_PrecType, Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Integer), c_double, POINTER(Complex), Integer, c_double, Integer, POINTER(Complex), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f11dsc(method, precon, n, nnz, a, irow, icol, omega, b, m, tol, maxitn, x, rnorm, itn, fail):
    """
    Solution of complex sparse non-Hermitian linear system, RGMRES, CGS, Bi-
    CGSTAB or TFQMR method, Jacobi or SSOR preconditioner Black Box.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, itn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11dsc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dsc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dsc', icol, 'icol')
    _b = nag_complex_type_check_and_cast('f11dsc', b, 'b')
    _x = nag_complex_type_check_and_cast('f11dsc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11dsc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11dsc', itn, 'itn')
    f11dsc_ctypes(method, precon, n, nnz, _a, _irow, _icol, omega, _b, m, tol, maxitn, _x, _rnorm, _itn, fail)

f11dtc_ctypes = _library_cl.f11dtc
f11dtc_ctypes.restype = None
f11dtc_ctypes.argtypes = [Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(Nag_SparseNsym_Piv), POINTER(Nag_SparseNsym_Fact), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f11dtc(n, nnz, a, la, irow, icol, nb, istb, indb, lindb, lfill, dtol, pstrat, milu, ipivp, ipivq, istr, idiag, nnzc, npivm, fail):
    """
    Complex, sparse, non-Hermitian linear system, incomplete LU
    factorization of local or overlapping diagonal blocks.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dtc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, istb, indb, lfill, dtol, ipivp, ipivq, istr, idiag, nnzc, npivm, pstrat, milu
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dtol
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, istb, indb, lfill, ipivp, ipivq, istr, idiag, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    TypeError will be raised when the following arguments are not of data type
    Nag_SparseNsym_Piv:
        pstrat
    TypeError will be raised when the following arguments are not of data type
    Nag_SparseNsym_Piv:
        milu
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11dtc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dtc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dtc', icol, 'icol')
    _istb = nag_integer_type_check_and_cast('f11dtc', istb, 'istb')
    _indb = nag_integer_type_check_and_cast('f11dtc', indb, 'indb')
    _lfill = nag_integer_type_check_and_cast('f11dtc', lfill, 'lfill')
    _dtol = nag_double_type_check_and_cast('f11dtc', dtol, 'dtol')
    _pstrat = nag_enum_type_check_and_cast('f11dtc', pstrat, 'pstrat', Nag_SparseNsym_Piv)
    _milu = nag_enum_type_check_and_cast('f11dtc', milu, 'milu', Nag_SparseNsym_Fact)
    _ipivp = nag_integer_type_check_and_cast('f11dtc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11dtc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11dtc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11dtc', idiag, 'idiag')
    _nnzc = nag_integer_type_check_and_cast('f11dtc', nnzc, 'nnzc')
    _npivm = nag_integer_type_check_and_cast('f11dtc', npivm, 'npivm')
    f11dtc_ctypes(n, nnz, _a, la, _irow, _icol, nb, _istb, _indb, lindb, _lfill, _dtol, _pstrat, _milu, _ipivp, _ipivq, _istr, _idiag, _nnzc, _npivm, fail)

f11duc_ctypes = _library_cl.f11duc
f11duc_ctypes.restype = None
f11duc_ctypes.argtypes = [Nag_SparseNsym_Method, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Complex), Integer, c_double, Integer, POINTER(Complex), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f11duc(method, n, nnz, a, la, irow, icol, nb, istb, indb, lindb, ipivp, ipivq, istr, idiag, b, m, tol, maxitn, x, rnorm, itn, fail):
    """
    Solution of complex, sparse, non-Hermitian linear system, RGMRES, CGS,
    Bi-CGSTAB or TFQMR method, incomplete LU block diagonal preconditioner
    computed by f11dtc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11duc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, istb, indb, ipivp, ipivq, istr, idiag, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, istb, indb, ipivp, ipivq, istr, idiag, itn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11duc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11duc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11duc', icol, 'icol')
    _istb = nag_integer_type_check_and_cast('f11duc', istb, 'istb')
    _indb = nag_integer_type_check_and_cast('f11duc', indb, 'indb')
    _ipivp = nag_integer_type_check_and_cast('f11duc', ipivp, 'ipivp')
    _ipivq = nag_integer_type_check_and_cast('f11duc', ipivq, 'ipivq')
    _istr = nag_integer_type_check_and_cast('f11duc', istr, 'istr')
    _idiag = nag_integer_type_check_and_cast('f11duc', idiag, 'idiag')
    _b = nag_complex_type_check_and_cast('f11duc', b, 'b')
    _x = nag_complex_type_check_and_cast('f11duc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11duc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11duc', itn, 'itn')
    f11duc_ctypes(method, n, nnz, _a, la, _irow, _icol, nb, _istb, _indb, lindb, _ipivp, _ipivq, _istr, _idiag, _b, m, tol, maxitn, _x, _rnorm, _itn, fail)

f11dxc_ctypes = _library_cl.f11dxc
f11dxc_ctypes.restype = None
f11dxc_ctypes.argtypes = [Nag_SparseNsym_Store, Nag_TransType, Nag_InitializeA, Integer, Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_CheckData, POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f11dxc(store, trans, init, niter, n, nnz, a, irow, icol, check, b, x, diag, fail):
    """
    Complex, sparse, Hermitian or non-Hermitian, linear systems, line Jacobi
    preconditioner.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11dxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, b, x, diag
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, x, diag
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11dxc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11dxc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11dxc', icol, 'icol')
    _b = nag_complex_type_check_and_cast('f11dxc', b, 'b')
    _x = nag_complex_type_check_and_cast('f11dxc', x, 'x')
    _diag = nag_complex_type_check_and_cast('f11dxc', diag, 'diag')
    f11dxc_ctypes(store, trans, init, niter, n, nnz, _a, _irow, _icol, check, _b, _x, _diag, fail)

f11gdc_ctypes = _library_cl.f11gdc
f11gdc_ctypes.restype = None
f11gdc_ctypes.argtypes = [Nag_SparseSym_Method, Nag_SparseSym_PrecType, Nag_SparseSym_Bisection, Nag_NormType, Nag_SparseSym_Weight, Integer, Integer, c_double, Integer, c_double, c_double, c_double, Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(NagError)]


def f11gdc(method, precon, sigcmp, norm, weight, iterm, n, tol, maxitn, anorm, sigmax, sigtol, maxits, monit, lwreq, work, lwork, fail):
    """
    Real sparse symmetric linear systems, setup for f11gec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11gdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lwreq, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        lwreq
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lwreq = nag_integer_type_check_and_cast('f11gdc', lwreq, 'lwreq')
    _work = nag_double_type_check_and_cast('f11gdc', work, 'work')
    f11gdc_ctypes(method, precon, sigcmp, norm, weight, iterm, n, tol, maxitn, anorm, sigmax, sigtol, maxits, monit, _lwreq, _work, lwork, fail)

f11gec_ctypes = _library_cl.f11gec
f11gec_ctypes.restype = None
f11gec_ctypes.argtypes = [POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f11gec(irevcm, u, v, wgt, work, lwork, fail):
    """
    Real sparse symmetric linear systems, preconditioned conjugate gradient
    or Lanczos method or the MINRES algorithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11gec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, u, v, wgt, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        u, v, wgt, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f11gec', irevcm, 'irevcm')
    _u = nag_double_type_check_and_cast('f11gec', u, 'u')
    _v = nag_double_type_check_and_cast('f11gec', v, 'v')
    _wgt = nag_double_type_check_and_cast('f11gec', wgt, 'wgt')
    _work = nag_double_type_check_and_cast('f11gec', work, 'work')
    f11gec_ctypes(_irevcm, _u, _v, _wgt, _work, lwork, fail)

f11gfc_ctypes = _library_cl.f11gfc
f11gfc_ctypes.restype = None
f11gfc_ctypes.argtypes = [POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f11gfc(itn, stplhs, stprhs, anorm, sigmax, its, sigerr, work, lwork, fail):
    """
    Real sparse symmetric linear systems, diagnostic for f11gec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11gfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        itn, stplhs, stprhs, anorm, sigmax, its, sigerr, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        stplhs, stprhs, anorm, sigmax, sigerr, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        itn, its
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _itn = nag_integer_type_check_and_cast('f11gfc', itn, 'itn')
    _stplhs = nag_double_type_check_and_cast('f11gfc', stplhs, 'stplhs')
    _stprhs = nag_double_type_check_and_cast('f11gfc', stprhs, 'stprhs')
    _anorm = nag_double_type_check_and_cast('f11gfc', anorm, 'anorm')
    _sigmax = nag_double_type_check_and_cast('f11gfc', sigmax, 'sigmax')
    _its = nag_integer_type_check_and_cast('f11gfc', its, 'its')
    _sigerr = nag_double_type_check_and_cast('f11gfc', sigerr, 'sigerr')
    _work = nag_double_type_check_and_cast('f11gfc', work, 'work')
    f11gfc_ctypes(_itn, _stplhs, _stprhs, _anorm, _sigmax, _its, _sigerr, _work, lwork, fail)

f11grc_ctypes = _library_cl.f11grc
f11grc_ctypes.restype = None
f11grc_ctypes.argtypes = [Nag_SparseSym_Method, Nag_SparseSym_PrecType, Nag_SparseSym_Bisection, Nag_NormType, Nag_SparseSym_Weight, Integer, Integer, c_double, Integer, c_double, c_double, c_double, Integer, Integer, POINTER(Integer), POINTER(Complex), Integer, POINTER(NagError)]


def f11grc(method, precon, sigcmp, norm, weight, iterm, n, tol, maxitn, anorm, sigmax, sigtol, maxits, monit, lwreq, work, lwork, fail):
    """
    Complex sparse Hermitian linear systems, setup for f11gsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11grc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        lwreq, work
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        lwreq
    TypeError will be raised when the following arguments are not of data type
    Complex:
        work
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _lwreq = nag_integer_type_check_and_cast('f11grc', lwreq, 'lwreq')
    _work = nag_complex_type_check_and_cast('f11grc', work, 'work')
    f11grc_ctypes(method, precon, sigcmp, norm, weight, iterm, n, tol, maxitn, anorm, sigmax, sigtol, maxits, monit, _lwreq, _work, lwork, fail)

f11gsc_ctypes = _library_cl.f11gsc
f11gsc_ctypes.restype = None
f11gsc_ctypes.argtypes = [POINTER(Integer), POINTER(Complex), POINTER(Complex), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f11gsc(irevcm, u, v, wgt, work, lwork, fail):
    """
    Complex sparse Hermitian linear systems, preconditioned conjugate
    gradient or Lanczos.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11gsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, u, v, wgt, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        wgt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm
    TypeError will be raised when the following arguments are not of data type
    Complex:
        u, v, work
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f11gsc', irevcm, 'irevcm')
    _u = nag_complex_type_check_and_cast('f11gsc', u, 'u')
    _v = nag_complex_type_check_and_cast('f11gsc', v, 'v')
    _wgt = nag_double_type_check_and_cast('f11gsc', wgt, 'wgt')
    _work = nag_complex_type_check_and_cast('f11gsc', work, 'work')
    f11gsc_ctypes(_irevcm, _u, _v, _wgt, _work, lwork, fail)

f11gtc_ctypes = _library_cl.f11gtc
f11gtc_ctypes.restype = None
f11gtc_ctypes.argtypes = [POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Complex), Integer, POINTER(NagError)]


def f11gtc(itn, stplhs, stprhs, anorm, sigmax, its, sigerr, work, lwork, fail):
    """
    Complex sparse Hermitian linear systems, diagnostic for f11gsc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11gtc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        itn, stplhs, stprhs, anorm, sigmax, its, sigerr, work
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        stplhs, stprhs, anorm, sigmax, sigerr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        itn, its
    TypeError will be raised when the following arguments are not of data type
    Complex:
        work
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _itn = nag_integer_type_check_and_cast('f11gtc', itn, 'itn')
    _stplhs = nag_double_type_check_and_cast('f11gtc', stplhs, 'stplhs')
    _stprhs = nag_double_type_check_and_cast('f11gtc', stprhs, 'stprhs')
    _anorm = nag_double_type_check_and_cast('f11gtc', anorm, 'anorm')
    _sigmax = nag_double_type_check_and_cast('f11gtc', sigmax, 'sigmax')
    _its = nag_integer_type_check_and_cast('f11gtc', its, 'its')
    _sigerr = nag_double_type_check_and_cast('f11gtc', sigerr, 'sigerr')
    _work = nag_complex_type_check_and_cast('f11gtc', work, 'work')
    f11gtc_ctypes(_itn, _stplhs, _stprhs, _anorm, _sigmax, _its, _sigerr, _work, lwork, fail)

f11jac_ctypes = _library_cl.f11jac
f11jac_ctypes.restype = None
f11jac_ctypes.argtypes = [Integer, Integer, POINTER(POINTER(c_double)), POINTER(Integer), POINTER(POINTER(Integer)), POINTER(POINTER(Integer)), Integer, c_double, Nag_SparseSym_Fact, c_double, Nag_SparseSym_Piv, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Nag_Sparse_Comm), POINTER(NagError)]


def f11jac(n, nnz, a, la, irow, icol, lfill, dtol, mic, dscale, pstrat, ipiv, istr, nnzc, npivm, comm, fail):
    """
    Incomplete Cholesky factorization (symmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, la, irow, icol, ipiv, istr, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        la, irow, icol, ipiv, istr, nnzc, npivm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11jac', a, 'a')
    _la = nag_integer_type_check_and_cast('f11jac', la, 'la')
    _irow = nag_integer_type_check_and_cast('f11jac', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jac', icol, 'icol')
    _ipiv = nag_integer_type_check_and_cast('f11jac', ipiv, 'ipiv')
    _istr = nag_integer_type_check_and_cast('f11jac', istr, 'istr')
    _nnzc = nag_integer_type_check_and_cast('f11jac', nnzc, 'nnzc')
    _npivm = nag_integer_type_check_and_cast('f11jac', npivm, 'npivm')
    f11jac_ctypes(n, nnz, _a, _la, _irow, _icol, lfill, dtol, mic, dscale, pstrat, _ipiv, _istr, _nnzc, _npivm, comm, fail)

f11jbc_ctypes = _library_cl.f11jbc
f11jbc_ctypes.restype = None
f11jbc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), Nag_SparseSym_CheckData, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11jbc(n, a, la, irow, icol, ipiv, istr, check, y, x, fail):
    """
    Solution of linear system involving incomplete Cholesky preconditioning
    matrix generated by f11jac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipiv, istr, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipiv, istr
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11jbc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jbc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jbc', icol, 'icol')
    _ipiv = nag_integer_type_check_and_cast('f11jbc', ipiv, 'ipiv')
    _istr = nag_integer_type_check_and_cast('f11jbc', istr, 'istr')
    _y = nag_double_type_check_and_cast('f11jbc', y, 'y')
    _x = nag_double_type_check_and_cast('f11jbc', x, 'x')
    f11jbc_ctypes(n, _a, la, _irow, _icol, _ipiv, _istr, check, _y, _x, fail)

f11jcc_ctypes = _library_cl.f11jcc
f11jcc_ctypes.restype = None
f11jcc_ctypes.argtypes = [Nag_SparseSym_Method, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Nag_Sparse_Comm), POINTER(NagError)]


def f11jcc(method, n, nnz, a, la, irow, icol, ipiv, istr, b, tol, maxitn, x, rnorm, itn, comm, fail):
    """
    Solver with incomplete Cholesky preconditioning (symmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipiv, istr, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipiv, istr, itn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11jcc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jcc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jcc', icol, 'icol')
    _ipiv = nag_integer_type_check_and_cast('f11jcc', ipiv, 'ipiv')
    _istr = nag_integer_type_check_and_cast('f11jcc', istr, 'istr')
    _b = nag_double_type_check_and_cast('f11jcc', b, 'b')
    _x = nag_double_type_check_and_cast('f11jcc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11jcc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11jcc', itn, 'itn')
    f11jcc_ctypes(method, n, nnz, _a, la, _irow, _icol, _ipiv, _istr, _b, tol, maxitn, _x, _rnorm, _itn, comm, fail)

f11jdc_ctypes = _library_cl.f11jdc
f11jdc_ctypes.restype = None
f11jdc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), c_double, Nag_SparseSym_CheckData, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11jdc(n, nnz, a, irow, icol, rdiag, omega, check, y, x, fail):
    """
    Solution of linear system involving preconditioning matrix generated by
    applying SSOR to real sparse symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, rdiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, rdiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11jdc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jdc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jdc', icol, 'icol')
    _rdiag = nag_double_type_check_and_cast('f11jdc', rdiag, 'rdiag')
    _y = nag_double_type_check_and_cast('f11jdc', y, 'y')
    _x = nag_double_type_check_and_cast('f11jdc', x, 'x')
    f11jdc_ctypes(n, nnz, _a, _irow, _icol, _rdiag, omega, check, _y, _x, fail)

f11jec_ctypes = _library_cl.f11jec
f11jec_ctypes.restype = None
f11jec_ctypes.argtypes = [Nag_SparseSym_Method, Nag_SparseSym_PrecType, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), c_double, POINTER(c_double), c_double, Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(Nag_Sparse_Comm), POINTER(NagError)]


def f11jec(method, precon, n, nnz, a, irow, icol, omega, b, tol, maxitn, x, rnorm, itn, comm, fail):
    """
    Solver with Jacobi, SSOR, or no preconditioning (symmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, x, rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, itn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11jec', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jec', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jec', icol, 'icol')
    _b = nag_double_type_check_and_cast('f11jec', b, 'b')
    _x = nag_double_type_check_and_cast('f11jec', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11jec', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11jec', itn, 'itn')
    f11jec_ctypes(method, precon, n, nnz, _a, _irow, _icol, omega, _b, tol, maxitn, _x, _rnorm, _itn, comm, fail)

f11jnc_ctypes = _library_cl.f11jnc
f11jnc_ctypes.restype = None
f11jnc_ctypes.argtypes = [Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), Integer, c_double, Nag_SparseSym_Fact, c_double, Nag_SparseSym_Piv, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f11jnc(n, nnz, a, la, irow, icol, lfill, dtol, mic, dscale, pstrat, ipiv, istr, nnzc, npivm, fail):
    """
    Complex sparse Hermitian matrix, incomplete Cholesky factorization.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipiv, istr, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipiv, istr, nnzc, npivm
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11jnc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jnc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jnc', icol, 'icol')
    _ipiv = nag_integer_type_check_and_cast('f11jnc', ipiv, 'ipiv')
    _istr = nag_integer_type_check_and_cast('f11jnc', istr, 'istr')
    _nnzc = nag_integer_type_check_and_cast('f11jnc', nnzc, 'nnzc')
    _npivm = nag_integer_type_check_and_cast('f11jnc', npivm, 'npivm')
    f11jnc_ctypes(n, nnz, _a, la, _irow, _icol, lfill, dtol, mic, dscale, pstrat, _ipiv, _istr, _nnzc, _npivm, fail)

f11jpc_ctypes = _library_cl.f11jpc
f11jpc_ctypes.restype = None
f11jpc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), Nag_SparseSym_CheckData, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f11jpc(n, a, la, irow, icol, ipiv, istr, check, y, x, fail):
    """
    Solution of complex linear system involving incomplete Cholesky
    preconditioning matrix generated by f11jnc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipiv, istr, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipiv, istr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, y, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11jpc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jpc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jpc', icol, 'icol')
    _ipiv = nag_integer_type_check_and_cast('f11jpc', ipiv, 'ipiv')
    _istr = nag_integer_type_check_and_cast('f11jpc', istr, 'istr')
    _y = nag_complex_type_check_and_cast('f11jpc', y, 'y')
    _x = nag_complex_type_check_and_cast('f11jpc', x, 'x')
    f11jpc_ctypes(n, _a, la, _irow, _icol, _ipiv, _istr, check, _y, _x, fail)

f11jqc_ctypes = _library_cl.f11jqc
f11jqc_ctypes.restype = None
f11jqc_ctypes.argtypes = [Nag_SparseSym_Method, Integer, Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(Complex), c_double, Integer, POINTER(Complex), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f11jqc(method, n, nnz, a, la, irow, icol, ipiv, istr, b, tol, maxitn, x, rnorm, itn, fail):
    """
    Solution of complex sparse Hermitian linear system, conjugate
    gradient/Lanczos method, preconditioner computed by f11jnc (Black Box).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, ipiv, istr, b, x, rnorm, itn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, ipiv, istr, itn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11jqc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jqc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jqc', icol, 'icol')
    _ipiv = nag_integer_type_check_and_cast('f11jqc', ipiv, 'ipiv')
    _istr = nag_integer_type_check_and_cast('f11jqc', istr, 'istr')
    _b = nag_complex_type_check_and_cast('f11jqc', b, 'b')
    _x = nag_complex_type_check_and_cast('f11jqc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11jqc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11jqc', itn, 'itn')
    f11jqc_ctypes(method, n, nnz, _a, la, _irow, _icol, _ipiv, _istr, _b, tol, maxitn, _x, _rnorm, _itn, fail)

f11jrc_ctypes = _library_cl.f11jrc
f11jrc_ctypes.restype = None
f11jrc_ctypes.argtypes = [Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Integer), POINTER(c_double), c_double, Nag_SparseSym_CheckData, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f11jrc(n, nnz, a, irow, icol, rdiag, omega, check, y, x, fail):
    """
    Solution of linear system involving preconditioning matrix generated by
    applying SSOR to complex sparse Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jrc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, rdiag, y, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rdiag
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, y, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11jrc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jrc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jrc', icol, 'icol')
    _rdiag = nag_double_type_check_and_cast('f11jrc', rdiag, 'rdiag')
    _y = nag_complex_type_check_and_cast('f11jrc', y, 'y')
    _x = nag_complex_type_check_and_cast('f11jrc', x, 'x')
    f11jrc_ctypes(n, nnz, _a, _irow, _icol, _rdiag, omega, check, _y, _x, fail)

f11jsc_ctypes = _library_cl.f11jsc
f11jsc_ctypes.restype = None
f11jsc_ctypes.argtypes = [Nag_SparseSym_Method, Nag_SparseSym_PrecType, Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Integer), c_double, POINTER(Complex), c_double, Integer, POINTER(Complex), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f11jsc(method, precon, n, nnz, a, irow, icol, omega, b, tol, maxitn, x, rnorm, itn, rdiag, fail):
    """
    Solution of complex sparse Hermitian linear system, conjugate
    gradient/Lanczos method, Jacobi or SSOR preconditioner (Black Box).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11jsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, b, x, rnorm, itn, rdiag
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rnorm, rdiag
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, itn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11jsc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11jsc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11jsc', icol, 'icol')
    _b = nag_complex_type_check_and_cast('f11jsc', b, 'b')
    _x = nag_complex_type_check_and_cast('f11jsc', x, 'x')
    _rnorm = nag_double_type_check_and_cast('f11jsc', rnorm, 'rnorm')
    _itn = nag_integer_type_check_and_cast('f11jsc', itn, 'itn')
    _rdiag = nag_double_type_check_and_cast('f11jsc', rdiag, 'rdiag')
    f11jsc_ctypes(method, precon, n, nnz, _a, _irow, _icol, omega, _b, tol, maxitn, _x, _rnorm, _itn, _rdiag, fail)

f11mdc_ctypes = _library_cl.f11mdc
f11mdc_ctypes.restype = None
f11mdc_ctypes.argtypes = [Nag_ColumnPermutationType, Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f11mdc(spec, n, icolzp, irowix, iprm, fail):
    """
    Real sparse nonsymmetric linear systems, setup for f11mec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icolzp, irowix, iprm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icolzp, irowix, iprm
    """
    from .util import nag_integer_type_check_and_cast
    _icolzp = nag_integer_type_check_and_cast('f11mdc', icolzp, 'icolzp')
    _irowix = nag_integer_type_check_and_cast('f11mdc', irowix, 'irowix')
    _iprm = nag_integer_type_check_and_cast('f11mdc', iprm, 'iprm')
    f11mdc_ctypes(spec, n, _icolzp, _irowix, _iprm, fail)

f11mec_ctypes = _library_cl.f11mec
f11mec_ctypes.restype = None
f11mec_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), c_double, Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f11mec(n, irowix, a, iprm, thresh, nzlmx, nzlumx, nzumx, il, lval, iu, uval, nnzl, nnzu, flop, fail):
    """
    LU factorization of real sparse matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irowix, a, iprm, nzlumx, il, lval, iu, uval, nnzl, nnzu, flop
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, lval, uval, flop
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irowix, iprm, nzlumx, il, iu, nnzl, nnzu
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irowix = nag_integer_type_check_and_cast('f11mec', irowix, 'irowix')
    _a = nag_double_type_check_and_cast('f11mec', a, 'a')
    _iprm = nag_integer_type_check_and_cast('f11mec', iprm, 'iprm')
    _nzlumx = nag_integer_type_check_and_cast('f11mec', nzlumx, 'nzlumx')
    _il = nag_integer_type_check_and_cast('f11mec', il, 'il')
    _lval = nag_double_type_check_and_cast('f11mec', lval, 'lval')
    _iu = nag_integer_type_check_and_cast('f11mec', iu, 'iu')
    _uval = nag_double_type_check_and_cast('f11mec', uval, 'uval')
    _nnzl = nag_integer_type_check_and_cast('f11mec', nnzl, 'nnzl')
    _nnzu = nag_integer_type_check_and_cast('f11mec', nnzu, 'nnzu')
    _flop = nag_double_type_check_and_cast('f11mec', flop, 'flop')
    f11mec_ctypes(n, _irowix, _a, _iprm, thresh, nzlmx, _nzlumx, nzumx, _il, _lval, _iu, _uval, _nnzl, _nnzu, _flop, fail)

f11mfc_ctypes = _library_cl.f11mfc
f11mfc_ctypes.restype = None
f11mfc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f11mfc(order, trans, n, iprm, il, lval, iu, uval, nrhs, b, pdb, fail):
    """
    Solution of real sparse simultaneous linear equations (coefficient
    matrix already factorized).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iprm, il, lval, iu, uval, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        lval, uval, b
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iprm, il, iu
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iprm = nag_integer_type_check_and_cast('f11mfc', iprm, 'iprm')
    _il = nag_integer_type_check_and_cast('f11mfc', il, 'il')
    _lval = nag_double_type_check_and_cast('f11mfc', lval, 'lval')
    _iu = nag_integer_type_check_and_cast('f11mfc', iu, 'iu')
    _uval = nag_double_type_check_and_cast('f11mfc', uval, 'uval')
    _b = nag_double_type_check_and_cast('f11mfc', b, 'b')
    f11mfc_ctypes(order, trans, n, _iprm, _il, _lval, _iu, _uval, nrhs, _b, pdb, fail)

f11mgc_ctypes = _library_cl.f11mgc
f11mgc_ctypes.restype = None
f11mgc_ctypes.argtypes = [Nag_NormType, Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def f11mgc(norm, n, il, lval, iu, uval, anorm, rcond, fail):
    """
    Estimate condition number of real matrix, matrix already factorized by
    f11mec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        il, lval, iu, uval, rcond
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        lval, uval, rcond
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        il, iu
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _il = nag_integer_type_check_and_cast('f11mgc', il, 'il')
    _lval = nag_double_type_check_and_cast('f11mgc', lval, 'lval')
    _iu = nag_integer_type_check_and_cast('f11mgc', iu, 'iu')
    _uval = nag_double_type_check_and_cast('f11mgc', uval, 'uval')
    _rcond = nag_double_type_check_and_cast('f11mgc', rcond, 'rcond')
    f11mgc_ctypes(norm, n, _il, _lval, _iu, _uval, anorm, _rcond, fail)

f11mhc_ctypes = _library_cl.f11mhc
f11mhc_ctypes.restype = None
f11mhc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11mhc(order, trans, n, icolzp, irowix, a, iprm, il, lval, iu, uval, nrhs, b, pdb, x, pdx, ferr, berr, fail):
    """
    Refined solution with error bounds of real system of linear equations,
    multiple right-hand sides.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icolzp, irowix, a, iprm, il, lval, iu, uval, b, x, ferr, berr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, lval, uval, b, x, ferr, berr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icolzp, irowix, iprm, il, iu
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _icolzp = nag_integer_type_check_and_cast('f11mhc', icolzp, 'icolzp')
    _irowix = nag_integer_type_check_and_cast('f11mhc', irowix, 'irowix')
    _a = nag_double_type_check_and_cast('f11mhc', a, 'a')
    _iprm = nag_integer_type_check_and_cast('f11mhc', iprm, 'iprm')
    _il = nag_integer_type_check_and_cast('f11mhc', il, 'il')
    _lval = nag_double_type_check_and_cast('f11mhc', lval, 'lval')
    _iu = nag_integer_type_check_and_cast('f11mhc', iu, 'iu')
    _uval = nag_double_type_check_and_cast('f11mhc', uval, 'uval')
    _b = nag_double_type_check_and_cast('f11mhc', b, 'b')
    _x = nag_double_type_check_and_cast('f11mhc', x, 'x')
    _ferr = nag_double_type_check_and_cast('f11mhc', ferr, 'ferr')
    _berr = nag_double_type_check_and_cast('f11mhc', berr, 'berr')
    f11mhc_ctypes(order, trans, n, _icolzp, _irowix, _a, _iprm, _il, _lval, _iu, _uval, nrhs, _b, pdb, _x, pdx, _ferr, _berr, fail)

f11mkc_ctypes = _library_cl.f11mkc
f11mkc_ctypes.restype = None
f11mkc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, c_double, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f11mkc(order, trans, n, m, alpha, icolzp, irowix, a, b, pdb, beta, c, pdc, fail):
    """
    Real sparse nonsymmetric matrix-matrix multiply, compressed column
    storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icolzp, irowix, a, b, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icolzp, irowix
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _icolzp = nag_integer_type_check_and_cast('f11mkc', icolzp, 'icolzp')
    _irowix = nag_integer_type_check_and_cast('f11mkc', irowix, 'irowix')
    _a = nag_double_type_check_and_cast('f11mkc', a, 'a')
    _b = nag_double_type_check_and_cast('f11mkc', b, 'b')
    _c = nag_double_type_check_and_cast('f11mkc', c, 'c')
    f11mkc_ctypes(order, trans, n, m, alpha, _icolzp, _irowix, _a, _b, pdb, beta, _c, pdc, fail)

f11mlc_ctypes = _library_cl.f11mlc
f11mlc_ctypes.restype = None
f11mlc_ctypes.argtypes = [Nag_NormType, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f11mlc(norm, anorm, n, icolzp, irowix, a, fail):
    """
    1-norm, infinity-norm, largest absolute element, real, square, sparse
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        anorm, icolzp, irowix, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        anorm, a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icolzp, irowix
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _anorm = nag_double_type_check_and_cast('f11mlc', anorm, 'anorm')
    _icolzp = nag_integer_type_check_and_cast('f11mlc', icolzp, 'icolzp')
    _irowix = nag_integer_type_check_and_cast('f11mlc', irowix, 'irowix')
    _a = nag_double_type_check_and_cast('f11mlc', a, 'a')
    f11mlc_ctypes(norm, _anorm, n, _icolzp, _irowix, _a, fail)

f11mmc_ctypes = _library_cl.f11mmc
f11mmc_ctypes.restype = None
f11mmc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11mmc(n, icolzp, a, iprm, il, lval, iu, uval, rpg, fail):
    """
    Real sparse nonsymmetric linear systems, diagnostic for f11mec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11mmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icolzp, a, iprm, il, lval, iu, uval, rpg
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, lval, uval, rpg
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icolzp, iprm, il, iu
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _icolzp = nag_integer_type_check_and_cast('f11mmc', icolzp, 'icolzp')
    _a = nag_double_type_check_and_cast('f11mmc', a, 'a')
    _iprm = nag_integer_type_check_and_cast('f11mmc', iprm, 'iprm')
    _il = nag_integer_type_check_and_cast('f11mmc', il, 'il')
    _lval = nag_double_type_check_and_cast('f11mmc', lval, 'lval')
    _iu = nag_integer_type_check_and_cast('f11mmc', iu, 'iu')
    _uval = nag_double_type_check_and_cast('f11mmc', uval, 'uval')
    _rpg = nag_double_type_check_and_cast('f11mmc', rpg, 'rpg')
    f11mmc_ctypes(n, _icolzp, _a, _iprm, _il, _lval, _iu, _uval, _rpg, fail)

f11xac_ctypes = _library_cl.f11xac
f11xac_ctypes.restype = None
f11xac_ctypes.argtypes = [Nag_TransType, Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_CheckData, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11xac(trans, n, nnz, a, irow, icol, check, x, y, fail):
    """
    Real, sparse, nonsymmetric matrix-vector multiply.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11xac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11xac', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11xac', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11xac', icol, 'icol')
    _x = nag_double_type_check_and_cast('f11xac', x, 'x')
    _y = nag_double_type_check_and_cast('f11xac', y, 'y')
    f11xac_ctypes(trans, n, nnz, _a, _irow, _icol, check, _x, _y, fail)

f11xec_ctypes = _library_cl.f11xec
f11xec_ctypes.restype = None
f11xec_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), Nag_SparseSym_CheckData, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f11xec(n, nnz, a, irow, icol, check, x, y, fail):
    """
    Real sparse symmetric matrix-vector multiply.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11xec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f11xec', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11xec', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11xec', icol, 'icol')
    _x = nag_double_type_check_and_cast('f11xec', x, 'x')
    _y = nag_double_type_check_and_cast('f11xec', y, 'y')
    f11xec_ctypes(n, nnz, _a, _irow, _icol, check, _x, _y, fail)

f11xnc_ctypes = _library_cl.f11xnc
f11xnc_ctypes.restype = None
f11xnc_ctypes.argtypes = [Nag_TransType, Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_CheckData, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f11xnc(trans, n, nnz, a, irow, icol, check, x, y, fail):
    """
    Complex sparse non-Hermitian matrix-vector multiply.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11xnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, x, y
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11xnc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11xnc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11xnc', icol, 'icol')
    _x = nag_complex_type_check_and_cast('f11xnc', x, 'x')
    _y = nag_complex_type_check_and_cast('f11xnc', y, 'y')
    f11xnc_ctypes(trans, n, nnz, _a, _irow, _icol, check, _x, _y, fail)

f11xsc_ctypes = _library_cl.f11xsc
f11xsc_ctypes.restype = None
f11xsc_ctypes.argtypes = [Integer, Integer, POINTER(Complex), POINTER(Integer), POINTER(Integer), Nag_SparseSym_CheckData, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f11xsc(n, nnz, a, irow, icol, check, x, y, fail):
    """
    Complex sparse Hermitian matrix-vector multiply.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11xsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, x, y
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f11xsc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11xsc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11xsc', icol, 'icol')
    _x = nag_complex_type_check_and_cast('f11xsc', x, 'x')
    _y = nag_complex_type_check_and_cast('f11xsc', y, 'y')
    f11xsc_ctypes(n, nnz, _a, _irow, _icol, check, _x, _y, fail)

f11yec_ctypes = _library_cl.f11yec
f11yec_ctypes.restype = None
f11yec_ctypes.argtypes = [Integer, Integer, POINTER(Integer), POINTER(Integer), POINTER(Nag_Boolean), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f11yec(n, nnz, icolzp, irowix, lopts, mask, perm, info, fail):
    """
    Reverse Cuthill--McKee reordering of a sparse symmetric matrix in CCS
    format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11yec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        icolzp, irowix, mask, perm, info, lopts
    TypeError will be raised when the following arguments are not of data type
    Nag_Boolean:
        lopts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icolzp, irowix, mask, perm, info
    """
    from .util import nag_enum_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _icolzp = nag_integer_type_check_and_cast('f11yec', icolzp, 'icolzp')
    _irowix = nag_integer_type_check_and_cast('f11yec', irowix, 'irowix')
    _mask = nag_integer_type_check_and_cast('f11yec', mask, 'mask')
    _perm = nag_integer_type_check_and_cast('f11yec', perm, 'perm')
    _info = nag_integer_type_check_and_cast('f11yec', info, 'info')
    _lopts = nag_enum_type_check_and_cast('f11yec', lopts, 'lopts', Nag_Boolean)
    f11yec_ctypes(n, nnz, _icolzp, _irowix, _lopts, _mask, _perm, _info, fail)

nag_sparse_sym_rcm = f11yec

f11zac_ctypes = _library_cl.f11zac
f11zac_ctypes.restype = None
f11zac_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_Dups, Nag_SparseNsym_Zeros, POINTER(Integer), POINTER(NagError)]


def f11zac(n, nnz, a, irow, icol, dup, zero, istr, fail):
    """
    Sparse sort (nonsymmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11zac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nnz, a, irow, icol, istr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nnz, irow, icol, istr
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nnz = nag_integer_type_check_and_cast('f11zac', nnz, 'nnz')
    _a = nag_double_type_check_and_cast('f11zac', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11zac', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11zac', icol, 'icol')
    _istr = nag_integer_type_check_and_cast('f11zac', istr, 'istr')
    f11zac_ctypes(n, _nnz, _a, _irow, _icol, dup, zero, _istr, fail)

f11zbc_ctypes = _library_cl.f11zbc
f11zbc_ctypes.restype = None
f11zbc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), Nag_SparseSym_Dups, Nag_SparseSym_Zeros, POINTER(Integer), POINTER(NagError)]


def f11zbc(n, nnz, a, irow, icol, dup, zero, istr, fail):
    """
    Sparse sort (symmetric).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11zbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nnz, a, irow, icol, istr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nnz, irow, icol, istr
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nnz = nag_integer_type_check_and_cast('f11zbc', nnz, 'nnz')
    _a = nag_double_type_check_and_cast('f11zbc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11zbc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11zbc', icol, 'icol')
    _istr = nag_integer_type_check_and_cast('f11zbc', istr, 'istr')
    f11zbc_ctypes(n, _nnz, _a, _irow, _icol, dup, zero, _istr, fail)

f11znc_ctypes = _library_cl.f11znc
f11znc_ctypes.restype = None
f11znc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(Complex), POINTER(Integer), POINTER(Integer), Nag_SparseNsym_Dups, Nag_SparseNsym_Zeros, POINTER(Integer), POINTER(NagError)]


def f11znc(n, nnz, a, irow, icol, dup, zero, istr, fail):
    """
    Complex sparse non-Hermitian matrix reorder function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11znc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nnz, a, irow, icol, istr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nnz, irow, icol, istr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nnz = nag_integer_type_check_and_cast('f11znc', nnz, 'nnz')
    _a = nag_complex_type_check_and_cast('f11znc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11znc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11znc', icol, 'icol')
    _istr = nag_integer_type_check_and_cast('f11znc', istr, 'istr')
    f11znc_ctypes(n, _nnz, _a, _irow, _icol, dup, zero, _istr, fail)

f11zpc_ctypes = _library_cl.f11zpc
f11zpc_ctypes.restype = None
f11zpc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(Complex), POINTER(Integer), POINTER(Integer), Nag_SparseSym_Dups, Nag_SparseSym_Zeros, POINTER(Integer), POINTER(NagError)]


def f11zpc(n, nnz, a, irow, icol, dup, zero, istr, fail):
    """
    Complex sparse Hermitian matrix reorder function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f11/f11zpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nnz, a, irow, icol, istr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nnz, irow, icol, istr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nnz = nag_integer_type_check_and_cast('f11zpc', nnz, 'nnz')
    _a = nag_complex_type_check_and_cast('f11zpc', a, 'a')
    _irow = nag_integer_type_check_and_cast('f11zpc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f11zpc', icol, 'icol')
    _istr = nag_integer_type_check_and_cast('f11zpc', istr, 'istr')
    f11zpc_ctypes(n, _nnz, _a, _irow, _icol, dup, zero, _istr, fail)

nag_sparse_nsym_basic_setup = f11bdc
nag_sparse_nsym_basic_solver = f11bec
nag_sparse_nsym_basic_diagnostic = f11bfc
nag_sparse_nherm_basic_setup = f11brc
nag_sparse_nherm_basic_solver = f11bsc
nag_sparse_nherm_basic_diagnostic = f11btc
nag_sparse_nsym_fac = f11dac
nag_sparse_nsym_precon_ilu_solve = f11dbc
nag_sparse_nsym_fac_sol = f11dcc
nag_sparse_nsym_precon_ssor_solve = f11ddc
nag_sparse_nsym_sol = f11dec
nag_sparse_nsym_precon_bdilu = f11dfc
nag_sparse_nsym_precon_bdilu_solve = f11dgc
nag_sparse_nsym_jacobi = f11dkc
nag_sparse_nherm_fac = f11dnc
nag_sparse_nherm_precon_ilu_solve = f11dpc
nag_sparse_nherm_fac_sol = f11dqc
nag_sparse_nherm_precon_ssor_solve = f11drc
nag_sparse_nherm_sol = f11dsc
nag_sparse_nherm_precon_bdilu = f11dtc
nag_sparse_nherm_precon_bdilu_solve = f11duc
nag_sparse_nherm_jacobi = f11dxc
nag_sparse_sym_basic_setup = f11gdc
nag_sparse_sym_basic_solver = f11gec
nag_sparse_sym_basic_diagnostic = f11gfc
nag_sparse_herm_basic_setup = f11grc
nag_sparse_herm_basic_solver = f11gsc
nag_sparse_herm_basic_diagnostic = f11gtc
nag_sparse_sym_chol_fac = f11jac
nag_sparse_sym_precon_ichol_solve = f11jbc
nag_sparse_sym_chol_sol = f11jcc
nag_sparse_sym_precon_ssor_solve = f11jdc
nag_sparse_sym_sol = f11jec
nag_sparse_herm_chol_fac = f11jnc
nag_sparse_herm_precon_ichol_solve = f11jpc
nag_sparse_herm_chol_sol = f11jqc
nag_sparse_herm_precon_ssor_solve = f11jrc
nag_sparse_herm_sol = f11jsc
nag_superlu_column_permutation = f11mdc
nag_superlu_lu_factorize = f11mec
nag_superlu_solve_lu = f11mfc
nag_superlu_condition_number_lu = f11mgc
nag_superlu_refine_lu = f11mhc
nag_superlu_matrix_product = f11mkc
nag_superlu_matrix_norm = f11mlc
nag_superlu_diagnostic_lu = f11mmc
nag_sparse_nsym_matvec = f11xac
nag_sparse_sym_matvec = f11xec
nag_sparse_nherm_matvec = f11xnc
nag_sparse_herm_matvec = f11xsc
nag_sparse_nsym_sort = f11zac
nag_sparse_sym_sort = f11zbc
nag_sparse_nherm_sort = f11znc
nag_sparse_herm_sort = f11zpc
__all__ = ['f11bdc', 'f11bec', 'f11bfc', 'f11brc', 'f11bsc', 'f11btc',
           'f11dac', 'f11dbc', 'f11dcc', 'f11ddc', 'f11dec', 'f11dfc',
           'f11dgc', 'f11dkc', 'f11dnc', 'f11dpc', 'f11dqc', 'f11drc',
           'f11dsc', 'f11dtc', 'f11duc', 'f11dxc', 'f11gdc', 'f11gec',
           'f11gfc', 'f11grc', 'f11gsc', 'f11gtc', 'f11jac', 'f11jbc',
           'f11jcc', 'f11jdc', 'f11jec', 'f11jnc', 'f11jpc', 'f11jqc',
           'f11jrc', 'f11jsc', 'f11mdc', 'f11mec', 'f11mfc', 'f11mgc',
           'f11mhc', 'f11mkc', 'f11mlc', 'f11mmc', 'f11xac', 'f11xec',
           'f11xnc', 'f11xsc', 'f11zac', 'f11zbc', 'f11znc', 'f11zpc',
           'nag_sparse_herm_basic_diagnostic',
           'nag_sparse_herm_basic_setup',
           'nag_sparse_herm_basic_solver', 'nag_sparse_herm_chol_fac',
           'nag_sparse_herm_chol_sol', 'nag_sparse_herm_matvec',
           'nag_sparse_herm_precon_ichol_solve',
           'nag_sparse_herm_precon_ssor_solve', 'nag_sparse_herm_sol',
           'nag_sparse_herm_sort',
           'nag_sparse_nherm_basic_diagnostic',
           'nag_sparse_nherm_basic_setup',
           'nag_sparse_nherm_basic_solver', 'nag_sparse_nherm_fac',
           'nag_sparse_nherm_fac_sol', 'nag_sparse_nherm_jacobi',
           'nag_sparse_nherm_matvec', 'nag_sparse_nherm_precon_bdilu',
           'nag_sparse_nherm_precon_bdilu_solve',
           'nag_sparse_nherm_precon_ilu_solve',
           'nag_sparse_nherm_precon_ssor_solve',
           'nag_sparse_nherm_sol', 'nag_sparse_nherm_sort',
           'nag_sparse_nsym_basic_diagnostic',
           'nag_sparse_nsym_basic_setup',
           'nag_sparse_nsym_basic_solver', 'nag_sparse_nsym_fac',
           'nag_sparse_nsym_fac_sol', 'nag_sparse_nsym_jacobi',
           'nag_sparse_nsym_matvec', 'nag_sparse_nsym_precon_bdilu',
           'nag_sparse_nsym_precon_bdilu_solve',
           'nag_sparse_nsym_precon_ilu_solve',
           'nag_sparse_nsym_precon_ssor_solve', 'nag_sparse_nsym_sol',
           'nag_sparse_nsym_sort', 'nag_sparse_sym_basic_diagnostic',
           'nag_sparse_sym_basic_setup',
           'nag_sparse_sym_basic_solver', 'nag_sparse_sym_chol_fac',
           'nag_sparse_sym_chol_sol', 'nag_sparse_sym_matvec',
           'nag_sparse_sym_precon_ichol_solve',
           'nag_sparse_sym_precon_ssor_solve', 'nag_sparse_sym_sol',
           'nag_sparse_sym_sort', 'nag_superlu_column_permutation',
           'nag_superlu_condition_number_lu',
           'nag_superlu_diagnostic_lu', 'nag_superlu_lu_factorize',
           'nag_superlu_matrix_norm', 'nag_superlu_matrix_product',
           'nag_superlu_refine_lu', 'nag_superlu_solve_lu']
