"""
Deprecated wrappers for the NAG C Library (Mark 26.0) g03 Chapter.

g03 - Multivariate Methods.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/G03/g03conts.html

NAG Copyright 2017.
"""
from ctypes import c_char_p
from ctypes import c_double
from ctypes import POINTER

from .util import get_library
from .util import Nag_PrinCompMat
from .util import Nag_PrinCompScores
from .util import NagError

from .util import Nag_Weightstype
from .util import Nag_RotationLoading
from .util import Nag_TransNorm
from .util import Nag_RotationScale
from .util import Nag_FacMat
from .util import Nag_E04_Opt
from .util import Nag_FacScoreMethod
from .util import Nag_FacRotation
from .util import Nag_GroupCovars
from .util import Nag_MahalDist
from .util import Nag_DiscrimMethod
from .util import Nag_PriorProbability
from .util import Nag_Boolean
from .util import Nag_MatUpdate
from .util import Nag_DistanceType
from .util import Nag_VarScaleType
from .util import Nag_ClusterMethod
from .util import Nag_DendOrient
from .util import Integer
from .util import Nag_Eigenvalues
from .util import Nag_ScaleCriterion
from .util import Nag_VarCovar

_library_cl = get_library()

STRING = c_char_p

g03aac_ctypes = _library_cl.g03aac
g03aac_ctypes.restype = None
g03aac_ctypes.argtypes = [Nag_PrinCompMat, Nag_PrinCompScores, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g03aac(pcmatrix, scores, n, m, x, tdx, isx, s, wt, nvar, e, tde, p, tdp, v, tdv, fail):
    """
    Principal component analysis.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, s, wt, e, p, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, s, wt, e, p, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03aac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03aac', isx, 'isx')
    _s = nag_double_type_check_and_cast('g03aac', s, 's')
    _wt = nag_double_type_check_and_cast('g03aac', wt, 'wt')
    _e = nag_double_type_check_and_cast('g03aac', e, 'e')
    _p = nag_double_type_check_and_cast('g03aac', p, 'p')
    _v = nag_double_type_check_and_cast('g03aac', v, 'v')
    g03aac_ctypes(pcmatrix, scores, n, m, _x, tdx, _isx, _s, _wt, nvar, _e, tde, _p, tdp, _v, tdv, fail)

g03acc_ctypes = _library_cl.g03acc
g03acc_ctypes.restype = None
g03acc_ctypes.argtypes = [Nag_Weightstype, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, c_double, POINTER(Integer), POINTER(NagError)]


def g03acc(weight, n, m, x, tdx, isx, nx, ing, ng, wt, nig, cvm, tdcvm, e, tde, ncv, cvx, tdcvx, tol, irankx, fail):
    """
    Canonical variate analysis.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03acc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, ing, wt, nig, cvm, e, ncv, cvx, irankx
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, wt, cvm, e, cvx
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, ing, nig, ncv, irankx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03acc', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03acc', isx, 'isx')
    _ing = nag_integer_type_check_and_cast('g03acc', ing, 'ing')
    _wt = nag_double_type_check_and_cast('g03acc', wt, 'wt')
    _nig = nag_integer_type_check_and_cast('g03acc', nig, 'nig')
    _cvm = nag_double_type_check_and_cast('g03acc', cvm, 'cvm')
    _e = nag_double_type_check_and_cast('g03acc', e, 'e')
    _ncv = nag_integer_type_check_and_cast('g03acc', ncv, 'ncv')
    _cvx = nag_double_type_check_and_cast('g03acc', cvx, 'cvx')
    _irankx = nag_integer_type_check_and_cast('g03acc', irankx, 'irankx')
    g03acc_ctypes(weight, n, m, _x, tdx, _isx, nx, _ing, ng, _wt, _nig, _cvm, tdcvm, _e, tde, _ncv, _cvx, tdcvx, tol, _irankx, fail)

g03adc_ctypes = _library_cl.g03adc
g03adc_ctypes.restype = None
g03adc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(NagError)]


def g03adc(n, m, z, tdz, isz, nx, ny, wt, e, tde, ncv, cvx, tdcvx, cvy, tdcvy, tol, fail):
    """
    Canonical correlation analysis.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03adc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        z, isz, wt, e, ncv, cvx, cvy
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        z, wt, e, cvx, cvy
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isz, ncv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _z = nag_double_type_check_and_cast('g03adc', z, 'z')
    _isz = nag_integer_type_check_and_cast('g03adc', isz, 'isz')
    _wt = nag_double_type_check_and_cast('g03adc', wt, 'wt')
    _e = nag_double_type_check_and_cast('g03adc', e, 'e')
    _ncv = nag_integer_type_check_and_cast('g03adc', ncv, 'ncv')
    _cvx = nag_double_type_check_and_cast('g03adc', cvx, 'cvx')
    _cvy = nag_double_type_check_and_cast('g03adc', cvy, 'cvy')
    g03adc_ctypes(n, m, _z, tdz, _isz, nx, ny, _wt, _e, tde, _ncv, _cvx, tdcvx, _cvy, tdcvy, tol, fail)

g03bac_ctypes = _library_cl.g03bac
g03bac_ctypes.restype = None
g03bac_ctypes.argtypes = [Nag_RotationLoading, c_double, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, c_double, Integer, POINTER(Integer), POINTER(NagError)]


def g03bac(stand, g, nvar, k, fl, tdf, flr, r, tdr, acc, maxit, _iter, fail):
    """
    Orthogonal rotations for loading matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        fl, flr, r, _iter
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        fl, flr, r
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _fl = nag_double_type_check_and_cast('g03bac', fl, 'fl')
    _flr = nag_double_type_check_and_cast('g03bac', flr, 'flr')
    _r = nag_double_type_check_and_cast('g03bac', r, 'r')
    __iter = nag_integer_type_check_and_cast('g03bac', _iter, '_iter')
    g03bac_ctypes(stand, g, nvar, k, _fl, tdf, _flr, _r, tdr, acc, maxit, __iter, fail)

g03bcc_ctypes = _library_cl.g03bcc
g03bcc_ctypes.restype = None
g03bcc_ctypes.argtypes = [Nag_TransNorm, Nag_RotationScale, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g03bcc(stand, pscale, n, m, x, tdx, y, tdy, yhat, r, tdr, alpha, rss, res, fail):
    """
    Procrustes rotations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03bcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, yhat, r, alpha, rss, res
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, yhat, r, alpha, rss, res
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03bcc', x, 'x')
    _y = nag_double_type_check_and_cast('g03bcc', y, 'y')
    _yhat = nag_double_type_check_and_cast('g03bcc', yhat, 'yhat')
    _r = nag_double_type_check_and_cast('g03bcc', r, 'r')
    _alpha = nag_double_type_check_and_cast('g03bcc', alpha, 'alpha')
    _rss = nag_double_type_check_and_cast('g03bcc', rss, 'rss')
    _res = nag_double_type_check_and_cast('g03bcc', res, 'res')
    g03bcc_ctypes(stand, pscale, n, m, _x, tdx, _y, tdy, _yhat, _r, tdr, _alpha, _rss, _res, fail)

g03bdc_ctypes = _library_cl.g03bdc
g03bdc_ctypes.restype = None
g03bdc_ctypes.argtypes = [Nag_RotationLoading, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g03bdc(stand, n, m, x, pdx, ro, pdro, power, fp, pdfp, r, pdr, phi, pdphi, fs, pdfs, fail):
    """
    ProMax rotations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03bdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ro, fp, r, phi, fs
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, ro, fp, r, phi, fs
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03bdc', x, 'x')
    _ro = nag_double_type_check_and_cast('g03bdc', ro, 'ro')
    _fp = nag_double_type_check_and_cast('g03bdc', fp, 'fp')
    _r = nag_double_type_check_and_cast('g03bdc', r, 'r')
    _phi = nag_double_type_check_and_cast('g03bdc', phi, 'phi')
    _fs = nag_double_type_check_and_cast('g03bdc', fs, 'fs')
    g03bdc_ctypes(stand, n, m, _x, pdx, _ro, pdro, power, _fp, pdfp, _r, pdr, _phi, pdphi, _fs, pdfs, fail)

g03cac_ctypes = _library_cl.g03cac
g03cac_ctypes.restype = None
g03cac_ctypes.argtypes = [Nag_FacMat, Integer, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_E04_Opt), c_double, POINTER(NagError)]


def g03cac(matrix, n, m, x, tdx, nvar, isx, nfac, wt, e, stat, com, psi, res, fl, tdfl, options, eps, fail):
    """
    Maximum likelihood estimates of parameters.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, wt, e, stat, com, psi, res, fl
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, wt, e, stat, com, psi, res, fl
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03cac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03cac', isx, 'isx')
    _wt = nag_double_type_check_and_cast('g03cac', wt, 'wt')
    _e = nag_double_type_check_and_cast('g03cac', e, 'e')
    _stat = nag_double_type_check_and_cast('g03cac', stat, 'stat')
    _com = nag_double_type_check_and_cast('g03cac', com, 'com')
    _psi = nag_double_type_check_and_cast('g03cac', psi, 'psi')
    _res = nag_double_type_check_and_cast('g03cac', res, 'res')
    _fl = nag_double_type_check_and_cast('g03cac', fl, 'fl')
    g03cac_ctypes(matrix, n, m, _x, tdx, nvar, _isx, nfac, _wt, _e, _stat, _com, _psi, _res, _fl, tdfl, options, eps, fail)

g03ccc_ctypes = _library_cl.g03ccc
g03ccc_ctypes.restype = None
g03ccc_ctypes.argtypes = [Nag_FacScoreMethod, Nag_FacRotation, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g03ccc(method, rotate, nvar, nfac, fl, tdfl, psi, e, r, tdr, fs, tdfs, fail):
    """
    Factor score coefficients, following g03cac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03ccc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        fl, psi, e, r, fs
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        fl, psi, e, r, fs
    """
    from .util import nag_double_type_check_and_cast
    _fl = nag_double_type_check_and_cast('g03ccc', fl, 'fl')
    _psi = nag_double_type_check_and_cast('g03ccc', psi, 'psi')
    _e = nag_double_type_check_and_cast('g03ccc', e, 'e')
    _r = nag_double_type_check_and_cast('g03ccc', r, 'r')
    _fs = nag_double_type_check_and_cast('g03ccc', fs, 'fs')
    g03ccc_ctypes(method, rotate, nvar, nfac, _fl, tdfl, _psi, _e, _r, tdr, _fs, tdfs, fail)

g03dac_ctypes = _library_cl.g03dac
g03dac_ctypes.restype = None
g03dac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g03dac(n, m, x, tdx, isx, nvar, ing, ng, wt, nig, gmean, tdg, det, gc, stat, df, sig, fail):
    """
    Test for equality of within-group covariance matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, ing, wt, nig, gmean, det, gc, stat, df, sig
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, wt, gmean, det, gc, stat, df, sig
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, ing, nig
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03dac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03dac', isx, 'isx')
    _ing = nag_integer_type_check_and_cast('g03dac', ing, 'ing')
    _wt = nag_double_type_check_and_cast('g03dac', wt, 'wt')
    _nig = nag_integer_type_check_and_cast('g03dac', nig, 'nig')
    _gmean = nag_double_type_check_and_cast('g03dac', gmean, 'gmean')
    _det = nag_double_type_check_and_cast('g03dac', det, 'det')
    _gc = nag_double_type_check_and_cast('g03dac', gc, 'gc')
    _stat = nag_double_type_check_and_cast('g03dac', stat, 'stat')
    _df = nag_double_type_check_and_cast('g03dac', df, 'df')
    _sig = nag_double_type_check_and_cast('g03dac', sig, 'sig')
    g03dac_ctypes(n, m, _x, tdx, _isx, nvar, _ing, ng, _wt, _nig, _gmean, tdg, _det, _gc, _stat, _df, _sig, fail)

g03dbc_ctypes = _library_cl.g03dbc
g03dbc_ctypes.restype = None
g03dbc_ctypes.argtypes = [Nag_GroupCovars, Nag_MahalDist, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g03dbc(equal, mode, nvar, ng, gmean, tdg, gc, nobs, m, isx, x, tdx, d, tdd, fail):
    """
    Mahalanobis squared distances, following g03dac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03dbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        gmean, gc, isx, x, d
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        gmean, gc, x, d
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _gmean = nag_double_type_check_and_cast('g03dbc', gmean, 'gmean')
    _gc = nag_double_type_check_and_cast('g03dbc', gc, 'gc')
    _isx = nag_integer_type_check_and_cast('g03dbc', isx, 'isx')
    _x = nag_double_type_check_and_cast('g03dbc', x, 'x')
    _d = nag_double_type_check_and_cast('g03dbc', d, 'd')
    g03dbc_ctypes(equal, mode, nvar, ng, _gmean, tdg, _gc, nobs, m, _isx, _x, tdx, _d, tdd, fail)

g03dcc_ctypes = _library_cl.g03dcc
g03dcc_ctypes.restype = None
g03dcc_ctypes.argtypes = [Nag_DiscrimMethod, Nag_GroupCovars, Nag_PriorProbability, Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Integer), Nag_Boolean, POINTER(c_double), POINTER(NagError)]


def g03dcc(_type, equal, priors, nvar, ng, nig, gmean, tdg, gc, det, nobs, m, isx, x, tdx, prior, p, tdp, iag, atiq, ati, fail):
    """
    Allocates observations to groups, following g03dac.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nig, gmean, gc, det, isx, x, prior, p, iag, ati
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        gmean, gc, det, x, prior, p, ati
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nig, isx, iag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nig = nag_integer_type_check_and_cast('g03dcc', nig, 'nig')
    _gmean = nag_double_type_check_and_cast('g03dcc', gmean, 'gmean')
    _gc = nag_double_type_check_and_cast('g03dcc', gc, 'gc')
    _det = nag_double_type_check_and_cast('g03dcc', det, 'det')
    _isx = nag_integer_type_check_and_cast('g03dcc', isx, 'isx')
    _x = nag_double_type_check_and_cast('g03dcc', x, 'x')
    _prior = nag_double_type_check_and_cast('g03dcc', prior, 'prior')
    _p = nag_double_type_check_and_cast('g03dcc', p, 'p')
    _iag = nag_integer_type_check_and_cast('g03dcc', iag, 'iag')
    _ati = nag_double_type_check_and_cast('g03dcc', ati, 'ati')
    g03dcc_ctypes(_type, equal, priors, nvar, ng, _nig, _gmean, tdg, _gc, _det, nobs, m, _isx, _x, tdx, _prior, _p, tdp, _iag, atiq, _ati, fail)

g03eac_ctypes = _library_cl.g03eac
g03eac_ctypes.restype = None
g03eac_ctypes.argtypes = [Nag_MatUpdate, Nag_DistanceType, Nag_VarScaleType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g03eac(update, dist, scale, n, m, x, tdx, isx, s, d, fail):
    """
    Compute distance (dissimilarity) matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03eac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, s, d
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, s, d
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03eac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03eac', isx, 'isx')
    _s = nag_double_type_check_and_cast('g03eac', s, 's')
    _d = nag_double_type_check_and_cast('g03eac', d, 'd')
    g03eac_ctypes(update, dist, scale, n, m, _x, tdx, _isx, _s, _d, fail)

g03ecc_ctypes = _library_cl.g03ecc
g03ecc_ctypes.restype = None
g03ecc_ctypes.argtypes = [Nag_ClusterMethod, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g03ecc(method, n, d, ilc, iuc, cd, iord, dord, fail):
    """
    Hierarchical cluster analysis.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03ecc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, ilc, iuc, cd, iord, dord
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, cd, dord
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ilc, iuc, iord
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('g03ecc', d, 'd')
    _ilc = nag_integer_type_check_and_cast('g03ecc', ilc, 'ilc')
    _iuc = nag_integer_type_check_and_cast('g03ecc', iuc, 'iuc')
    _cd = nag_double_type_check_and_cast('g03ecc', cd, 'cd')
    _iord = nag_integer_type_check_and_cast('g03ecc', iord, 'iord')
    _dord = nag_double_type_check_and_cast('g03ecc', dord, 'dord')
    g03ecc_ctypes(method, n, _d, _ilc, _iuc, _cd, _iord, _dord, fail)

g03efc_ctypes = _library_cl.g03efc
g03efc_ctypes.restype = None
g03efc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g03efc(n, m, x, tdx, isx, nvar, k, cmeans, tdc, wt, inc, nic, css, csw, maxit, fail):
    """
    K-means.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03efc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, cmeans, wt, inc, nic, css, csw
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, cmeans, wt, css, csw
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, inc, nic
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03efc', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03efc', isx, 'isx')
    _cmeans = nag_double_type_check_and_cast('g03efc', cmeans, 'cmeans')
    _wt = nag_double_type_check_and_cast('g03efc', wt, 'wt')
    _inc = nag_integer_type_check_and_cast('g03efc', inc, 'inc')
    _nic = nag_integer_type_check_and_cast('g03efc', nic, 'nic')
    _css = nag_double_type_check_and_cast('g03efc', css, 'css')
    _csw = nag_double_type_check_and_cast('g03efc', csw, 'csw')
    g03efc_ctypes(n, m, _x, tdx, _isx, nvar, k, _cmeans, tdc, _wt, _inc, _nic, _css, _csw, maxit, fail)

g03ehc_ctypes = _library_cl.g03ehc
g03ehc_ctypes.restype = None
g03ehc_ctypes.argtypes = [Nag_DendOrient, Integer, POINTER(c_double), c_double, c_double, Integer, POINTER(POINTER(STRING)), POINTER(NagError)]


def g03ehc(orient, n, dord, dmin, dstep, nsym, c, fail):
    """
    Construct dendogram following g03ecc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03ehc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dord
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dord
    """
    from .util import nag_double_type_check_and_cast
    _dord = nag_double_type_check_and_cast('g03ehc', dord, 'dord')
    g03ehc_ctypes(orient, n, _dord, dmin, dstep, nsym, c, fail)

g03ejc_ctypes = _library_cl.g03ejc
g03ejc_ctypes.restype = None
g03ejc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g03ejc(n, cd, iord, dord, k, dlevel, ic, fail):
    """
    Construct clusters following g03ecc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03ejc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        cd, iord, dord, k, dlevel, ic
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        cd, dord, dlevel
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iord, k, ic
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _cd = nag_double_type_check_and_cast('g03ejc', cd, 'cd')
    _iord = nag_integer_type_check_and_cast('g03ejc', iord, 'iord')
    _dord = nag_double_type_check_and_cast('g03ejc', dord, 'dord')
    _k = nag_integer_type_check_and_cast('g03ejc', k, 'k')
    _dlevel = nag_double_type_check_and_cast('g03ejc', dlevel, 'dlevel')
    _ic = nag_integer_type_check_and_cast('g03ejc', ic, 'ic')
    g03ejc_ctypes(n, _cd, _iord, _dord, _k, _dlevel, _ic, fail)

g03fac_ctypes = _library_cl.g03fac
g03fac_ctypes.restype = None
g03fac_ctypes.argtypes = [Nag_Eigenvalues, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def g03fac(roots, n, d, ndim, x, tdx, _eval, fail):
    """
    Principal coordinate analysis.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03fac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, x, _eval
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, x, _eval
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('g03fac', d, 'd')
    _x = nag_double_type_check_and_cast('g03fac', x, 'x')
    __eval = nag_double_type_check_and_cast('g03fac', _eval, '_eval')
    g03fac_ctypes(roots, n, _d, ndim, _x, tdx, __eval, fail)

g03fcc_ctypes = _library_cl.g03fcc
g03fcc_ctypes.restype = None
g03fcc_ctypes.argtypes = [Nag_ScaleCriterion, Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_E04_Opt), POINTER(NagError)]


def g03fcc(_type, n, ndim, d, x, tdx, stress, dfit, options, fail):
    """
    Multidimensional scaling.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03fcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, x, stress, dfit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, x, stress, dfit
    """
    from .util import nag_double_type_check_and_cast
    _d = nag_double_type_check_and_cast('g03fcc', d, 'd')
    _x = nag_double_type_check_and_cast('g03fcc', x, 'x')
    _stress = nag_double_type_check_and_cast('g03fcc', stress, 'stress')
    _dfit = nag_double_type_check_and_cast('g03fcc', dfit, 'dfit')
    g03fcc_ctypes(_type, n, ndim, _d, _x, tdx, _stress, _dfit, options, fail)

g03gac_ctypes = _library_cl.g03gac
g03gac_ctypes.restype = None
g03gac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, Integer, Nag_Boolean, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), Nag_VarCovar, POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def g03gac(n, m, x, tdx, isx, nvar, ng, popt, prob, lprob, niter, riter, w, g, sopt, s, f, tol, loglik, fail):
    """
    Fits a Gaussian mixture model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, prob, niter, w, g, s, f, loglik
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, prob, w, g, s, f, loglik
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, niter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03gac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03gac', isx, 'isx')
    _prob = nag_double_type_check_and_cast('g03gac', prob, 'prob')
    _niter = nag_integer_type_check_and_cast('g03gac', niter, 'niter')
    _w = nag_double_type_check_and_cast('g03gac', w, 'w')
    _g = nag_double_type_check_and_cast('g03gac', g, 'g')
    _s = nag_double_type_check_and_cast('g03gac', s, 's')
    _f = nag_double_type_check_and_cast('g03gac', f, 'f')
    _loglik = nag_double_type_check_and_cast('g03gac', loglik, 'loglik')
    g03gac_ctypes(n, m, _x, tdx, _isx, nvar, ng, popt, _prob, lprob, _niter, riter, _w, _g, sopt, _s, _f, tol, _loglik, fail)

g03xzc_ctypes = _library_cl.g03xzc
g03xzc_ctypes.restype = None
g03xzc_ctypes.argtypes = [POINTER(POINTER(STRING))]


def g03xzc(c):
    """
    Frees memory allocated to the dendrogram array in g03ehc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03xzc.html
    """
    g03xzc_ctypes(c)

g03zac_ctypes = _library_cl.g03zac
g03zac_ctypes.restype = None
g03zac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g03zac(n, m, x, tdx, nvar, isx, s, e, z, tdz, fail):
    """
    Standardize values of a data matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g03/g03zac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, s, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, s, e, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g03zac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g03zac', isx, 'isx')
    _s = nag_double_type_check_and_cast('g03zac', s, 's')
    _e = nag_double_type_check_and_cast('g03zac', e, 'e')
    _z = nag_double_type_check_and_cast('g03zac', z, 'z')
    g03zac_ctypes(n, m, _x, tdx, nvar, _isx, _s, _e, _z, tdz, fail)

nag_mv_prin_comp = g03aac
nag_mv_canon_var = g03acc
nag_mv_canon_corr = g03adc
nag_mv_orthomax = g03bac
nag_mv_procustes = g03bcc
nag_mv_promax = g03bdc
nag_mv_factor = g03cac
nag_mv_fac_score = g03ccc
nag_mv_discrim = g03dac
nag_mv_discrim_mahaldist = g03dbc
nag_mv_discrim_group = g03dcc
nag_mv_distance_mat = g03eac
nag_mv_hierar_cluster_analysis = g03ecc
nag_mv_kmeans_cluster_analysis = g03efc
nag_mv_dendrogram = g03ehc
nag_mv_cluster_indicator = g03ejc
nag_mv_prin_coord_analysis = g03fac
nag_mv_ordinal_multidimscale = g03fcc
nag_mv_gaussian_mixture = g03gac
nag_mv_dend_free = g03xzc
nag_mv_z_scores = g03zac
__all__ = ['g03aac', 'g03acc', 'g03adc', 'g03bac', 'g03bcc', 'g03bdc',
           'g03cac', 'g03ccc', 'g03dac', 'g03dbc', 'g03dcc', 'g03eac',
           'g03ecc', 'g03efc', 'g03ehc', 'g03ejc', 'g03fac', 'g03fcc',
           'g03gac', 'g03xzc', 'g03zac', 'nag_mv_canon_corr',
           'nag_mv_canon_var', 'nag_mv_cluster_indicator',
           'nag_mv_dend_free', 'nag_mv_dendrogram', 'nag_mv_discrim',
           'nag_mv_discrim_group', 'nag_mv_discrim_mahaldist',
           'nag_mv_distance_mat', 'nag_mv_fac_score', 'nag_mv_factor',
           'nag_mv_gaussian_mixture',
           'nag_mv_hierar_cluster_analysis',
           'nag_mv_kmeans_cluster_analysis',
           'nag_mv_ordinal_multidimscale', 'nag_mv_orthomax',
           'nag_mv_prin_comp', 'nag_mv_prin_coord_analysis',
           'nag_mv_procustes', 'nag_mv_promax', 'nag_mv_z_scores']
